/*
 * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * Copyright (c) 1999 Apple Computer, Inc.  All Rights Reserved.
 * The contents of this file constitute Original Code as defined in and are 
 * subject to the Apple Public Source License Version 1.1 (the "License").  
 * You may not use this file except in compliance with the License.  Please 
 * obtain a copy of the License at http://www.apple.com/publicsource and 
 * read it before using this file.
 * 
 * This Original Code and all software distributed under the License are 
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, 
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS 
 * FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the License for 
 * the specific language governing rights and limitations under the 
 * License.
 * 
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
/*
	File:		RCFSourceInfo.h

	Contains:	This object takes input RCF data, and uses it to support the SourceInfo
				API.

	$Log: RCFSourceInfo.h,v $
	Revision 1.2  2000/01/27 02:07:45  rtucker
	Merge changes from v81
	
	Revision 1.4  2000/01/26 21:43:32  serenyi
	Fix 2399850 and now allow reflector to act as an RTSP client
	
	Revision 1.3  1999/12/21 01:43:25  lecroy
	*** empty log message ***
	
	Revision 1.2  1999/10/22 23:01:56  serenyi
	Made it so the reflector never reflects any more data after a TEARDOWN has been issued.
	
	Revision 1.1  1999/10/11 18:51:12  serenyi
	Created
	

*/

#ifndef __RCF_SOURCE_INFO_H__
#define __RCF_SOURCE_INFO_H__

#include "StrPtrLen.h"
#include "SourceInfo.h"
#include "FilePrefsSource.h"
#include "StringParser.h"

class RCFSourceInfo : public SourceInfo
{
	public:
	
		// Uses the SDP Data to build up the StreamInfo structures
		RCFSourceInfo() {}
		RCFSourceInfo(FilePrefsSource* inPrefsSource, UInt32 inStartIndex) { Parse(inPrefsSource, inStartIndex); }
		virtual ~RCFSourceInfo();
		
		// Parses out the SDP file provided, sets up the StreamInfo structures
		void	Parse(FilePrefsSource* inPrefsSource, SInt32 inStartIndex);
		
		// Parses relay_destination lines and builds OutputInfo structs
		void 	ParseRelayDestinations(FilePrefsSource* inPrefsSource, SInt32 inStartIndex);

};
#endif // __SDP_SOURCE_INFO_H__


