/*
 * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * Copyright (c) 1999 Apple Computer, Inc.  All Rights Reserved.
 * The contents of this file constitute Original Code as defined in and are 
 * subject to the Apple Public Source License Version 1.1 (the "License").  
 * You may not use this file except in compliance with the License.  Please 
 * obtain a copy of the License at http://www.apple.com/publicsource and 
 * read it before using this file.
 * 
 * This Original Code and all software distributed under the License are 
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, 
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS 
 * FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the License for 
 * the specific language governing rights and limitations under the 
 * License.
 * 
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
/*
	File:		RTSPReflectorOutput.h

	Contains:	Derived from ReflectorOutput, this implements the WritePacket
				method in terms of the QTSS API (that is, it writes to a client
				using the QTSS_RTPSessionObject
					
	$Log: RTPSessionOutput.h,v $
	Revision 1.1.1.1  2000/01/17 23:35:56  wsanchez
	Import of StreamingServer-2.0 (Apple)
	
	Revision 1.7  1999/12/21 01:43:26  lecroy
	*** empty log message ***
	
	Revision 1.6  1999/11/08 23:02:01  serenyi
	Updated to latest version of QTSS API
	
	Revision 1.5  1999/10/26 00:15:23  serenyi
	Make sure to check that there is no output for this session on a DESCRIBE
	
	Revision 1.4  1999/10/22 23:01:56  serenyi
	Made it so the reflector never reflects any more data after a TEARDOWN has been issued.
	
	Revision 1.3  1999/10/14 01:17:28  rtucker
	updates for thinning
	
	Revision 1.2  1999/10/07 09:01:49  serenyi
	Changed bool to Bool16, changed QTSS_Dictionary to QTSS_Object
	
	Revision 1.1.1.1  1999/09/22 23:01:08  lecroy
	importing
	
	Revision 1.1.1.1  1999/09/13 16:16:34  rtucker
	initial checkin from rrv14
	
	

*/

#ifndef __RTSP_REFLECTOR_OUTPUT_H__
#define __RTSP_REFLECTOR_OUTPUT_H__

#include "ReflectorOutput.h"
#include "ReflectorSession.h"
#include "QTSS.h"

class RTPSessionOutput : public ReflectorOutput
{
	public:
	
		// Adds some dictionary attributes
		static void Register();
		
		RTPSessionOutput(QTSS_ClientSessionObject inRTPSession, ReflectorSession* inReflectorSession, QTSS_Object serverPrefs);
		virtual ~RTPSessionOutput() {}
		
		ReflectorSession* GetReflectorSession() { return fReflectorSession; }
		
		// This writes the packet out to the proper QTSS_RTPStreamObject.
		virtual QTSS_Error	WritePacket(StrPtrLen* inPacket, UInt32 inStreamID, UInt32 inFlags, SInt64 packetLatenessInMSec );
		
	private:
	
		QTSS_ClientSessionObject fClientSession;
		ReflectorSession* 		fReflectorSession;
		
		// For use with thinning.
		SInt32		fMaxVideoDelayTolerance;
		SInt32		fMaxAudioDelayTolerance;
		
		UInt16 GetPacketSeqNumber(StrPtrLen* inPacket);
		void SetPacketSeqNumber(StrPtrLen* inPacket, UInt16 inSeqNumber);
		Bool16 PacketShouldBeThinned(QTSS_RTPStreamObject inStream, StrPtrLen* inPacket);

};

#endif //__RTSP_REFLECTOR_OUTPUT_H__
