/*
 * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * Copyright (c) 1999 Apple Computer, Inc.  All Rights Reserved.
 * The contents of this file constitute Original Code as defined in and are 
 * subject to the Apple Public Source License Version 1.1 (the "License").  
 * You may not use this file except in compliance with the License.  Please 
 * obtain a copy of the License at http://www.apple.com/publicsource and 
 * read it before using this file.
 * 
 * This Original Code and all software distributed under the License are 
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, 
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS 
 * FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the License for 
 * the specific language governing rights and limitations under the 
 * License.
 * 
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
/*
	File:		QTSS_Private.c

	Contains:	Code for stub library and stub callback functions.
					
	$Log: QTSS_Private.cpp,v $
	Revision 1.3  2000/03/09 22:52:17  serenyi
	Made work on freebsd
	
	Revision 1.2  2000/01/27 02:09:04  rtucker
	Merge changes from v81
	
	Revision 1.7  2000/01/26 21:34:47  serenyi
	Added QTSS_RequestIdleTime callback
	
	Revision 1.6  1999/12/21 01:43:30  lecroy
	*** empty log message ***
	
	Revision 1.5  1999/11/08 23:01:57  serenyi
	Updated to latest version of QTSS API
	
	Revision 1.4  1999/10/27 17:52:50  serenyi
	ntegrated Matthias's MacOS / GUSI changes
	
	Revision 1.3  1999/10/07 09:02:24  serenyi
	Changed bool to Bool16, changed QTSS_Dictionary to QTSS_Object
	
	Revision 1.2  1999/09/24 19:52:57  murata
	Added new dictionary fields for accessing Client Environment info. Also added new
	API function for converting qtss milliseconds to Unix seconds (1970 based).
	
	Revision 1.1.1.1  1999/09/22 23:01:15  lecroy
	importing
	
	Revision 1.1.1.1  1999/09/13 16:16:36  rtucker
	initial checkin from rrv14
	
	
	
*/

#include <stdlib.h>

#include <sys/types.h>
#include <sys/uio.h>

#include "QTSS.h"
#include "QTSS_Private.h"

static QTSS_CallbacksPtr	sCallbacks = NULL;
static QTSS_StreamRef		sErrorLogStream = NULL;

QTSS_Error _stublibrary_main(void* inPrivateArgs, QTSS_DispatchFuncPtr inDispatchFunc)
{
	QTSS_PrivateArgsPtr theArgs = (QTSS_PrivateArgsPtr)inPrivateArgs;
	
	// Setup

	sCallbacks = theArgs->inCallbacks;
	sErrorLogStream = theArgs->inErrorLogStream;
	
	// Send requested information back to the server
	
	theArgs->outStubLibraryVersion = QTSS_API_VERSION;
	theArgs->outDispatchFunction = inDispatchFunc;
	
	return QTSS_NoErr;
}

// STUB FUNCTION DEFINITIONS

void* 			QTSS_New(FourCharCode inMemoryIdentifier, UInt32 inSize)
{
	return (void *) ((QTSS_CallbackPtrProcPtr) sCallbacks->addr [kNewCallback]) (inMemoryIdentifier, inSize);
}

void			QTSS_Delete(void* inMemory)
{
	(sCallbacks->addr [kDeleteCallback]) (inMemory);
}

SInt64			QTSS_Milliseconds(void)
{
	SInt64 outMilliseconds = 0;
	(sCallbacks->addr [kMillisecondsCallback]) (&outMilliseconds);
	return outMilliseconds;
}

time_t			QTSS_MilliSecsTo1970Secs(SInt64 inQTSS_MilliSeconds)
{
	time_t outSeconds = 0;
	(sCallbacks->addr [kConvertToUnixTimeCallback]) (&inQTSS_MilliSeconds, &outSeconds);
	return outSeconds;
}

// STARTUP ROUTINES
	
QTSS_Error 	QTSS_AddRole(QTSS_Role inRole)
{
	return (sCallbacks->addr [kAddRoleCallback]) (inRole);	
}

// DICTIONARY ROUTINES

QTSS_Error	QTSS_AddAttribute(QTSS_ObjectType inType, const char* inTag, QTSS_AttrFunctionPtr inFunctionPtr)
{
	return (sCallbacks->addr [kAddAttributeCallback]) (inType, inTag, inFunctionPtr);	
}

QTSS_Error	QTSS_IDForAttr(QTSS_ObjectType inType, const char* inTag, QTSS_AttributeID* outID)
{
	return (sCallbacks->addr [kIDForTagCallback]) (inType, inTag, outID);	
}

QTSS_Error 	QTSS_GetValuePtr (QTSS_Object inDictionary, QTSS_AttributeID inID, UInt32 inIndex, void** outBuffer, UInt32* outLen)
{
	return (sCallbacks->addr [kGetAttributePtrByIDCallback]) (inDictionary, inID, inIndex, outBuffer, outLen);	
}

QTSS_Error 	QTSS_GetValue (QTSS_Object inDictionary, QTSS_AttributeID inID, UInt32 inIndex, void* ioBuffer, UInt32* ioLen)
{
	return (sCallbacks->addr [kGetAttributeByIDCallback]) (inDictionary, inID, inIndex, ioBuffer, ioLen);	
}

QTSS_Error 	QTSS_SetValue (QTSS_Object inDictionary, QTSS_AttributeID inID,UInt32 inIndex,  const void* inBuffer,  UInt32 inLen)
{
	return (sCallbacks->addr [kSetAttributeByIDCallback]) (inDictionary, inID, inIndex, inBuffer, inLen);	
}

// STREAM ROUTINES

QTSS_Error	QTSS_Write(QTSS_StreamRef inStream, const void* inBuffer, UInt32 inLen, UInt32* outLenWritten, UInt32 inFlags)
{
	return (sCallbacks->addr [kWriteCallback]) (inStream, inBuffer, inLen, outLenWritten, inFlags);	
}

QTSS_Error	QTSS_WriteV(QTSS_StreamRef inStream, iovec* inVec, UInt32 inNumVectors, UInt32 inTotalLength, UInt32* outLenWritten)
{
	return (sCallbacks->addr [kWriteVCallback]) (inStream, inVec, inNumVectors, inTotalLength, outLenWritten);	
}

QTSS_Error	QTSS_Flush(QTSS_StreamRef inStream)
{
	return (sCallbacks->addr [kFlushCallback]) (inStream);	
}

// SERVICE ROUTINES

QTSS_Error QTSS_AddService(const char* inServiceName, QTSS_ServiceFunctionPtr inFunctionPtr)
{
	return (sCallbacks->addr [kAddServiceCallback]) (inServiceName, inFunctionPtr);		
}

QTSS_Error QTSS_IDForService(const char* inTag, QTSS_ServiceID* outID)
{
	return (sCallbacks->addr [kIDForServiceCallback]) (inTag, outID);	
}

QTSS_Error QTSS_DoService(QTSS_ServiceID inID, QTSS_ServiceFunctionArgsPtr inArgs)
{
	return (sCallbacks->addr [kDoServiceCallback]) (inID, inArgs);	
}

// RTSP ROUTINES

QTSS_Error QTSS_SendRTSPHeaders(QTSS_RTSPRequestObject inRef)
{
	return (sCallbacks->addr [kSendRTSPHeadersCallback]) (inRef);		
}

QTSS_Error QTSS_AppendRTSPHeader(QTSS_RTSPRequestObject inRef, QTSS_RTSPHeader inHeader, const char* inValue, UInt32 inValueLen)
{
	return (sCallbacks->addr [kAppendRTSPHeadersCallback]) (inRef, inHeader, inValue, inValueLen);		
}

QTSS_Error QTSS_SendStandardRTSPResponse(QTSS_RTSPRequestObject inRTSPRequest, QTSS_Object inRTPInfo, UInt32 inFlags)
{
	return (sCallbacks->addr [kSendStandardRTSPCallback]) (inRTSPRequest, inRTPInfo, inFlags);		
}

// RTP ROUTINES

QTSS_Error QTSS_AddRTPStream(QTSS_ClientSessionObject inClientSession, QTSS_RTSPRequestObject inRTSPRequest, QTSS_RTPStreamObject* outStream, QTSS_AddStreamFlags inFlags)
{
	return (sCallbacks->addr [kAddRTPStreamCallback]) (inClientSession, inRTSPRequest, outStream, inFlags);		
}

QTSS_Error QTSS_Play(QTSS_ClientSessionObject inClientSession, QTSS_RTSPRequestObject inRTSPRequest, QTSS_PlayFlags inPlayFlags)
{
	return (sCallbacks->addr [kPlayCallback]) (inClientSession, inRTSPRequest, inPlayFlags);		
}

QTSS_Error QTSS_Pause(QTSS_ClientSessionObject inClientSession)
{
	return (sCallbacks->addr [kPauseCallback]) (inClientSession);		
}

QTSS_Error QTSS_Teardown(QTSS_ClientSessionObject inClientSession)
{
	return (sCallbacks->addr [kTeardownCallback]) (inClientSession);		
}

// ASYNC I/O ROUTINES

QTSS_Error	QTSS_CreateEventContext(int inFileDesc, QTSS_EventContextRef* outContext, void* inPrivate)
{
	return (sCallbacks->addr [kCreateEventContextCallback]) (inFileDesc, outContext, inPrivate);		
}

QTSS_Error	QTSS_DestroyEventContext(QTSS_EventContextRef inContext)
{
	return (sCallbacks->addr [kDestroyEventContextCallback]) (inContext);		
}

QTSS_Error  QTSS_RequestEvent(QTSS_EventContextRef inContext, int inEventMask)
{
	return (sCallbacks->addr [kRequestEventCallback]) (inContext, inEventMask);		
}
QTSS_Error  QTSS_SetIdleTimer(SInt64 inIdleMsec)
{
	return (sCallbacks->addr [kSetIdleTimerCallback]) (inIdleMsec);		
}
