/*
 * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * Copyright (c) 1999 Apple Computer, Inc.  All Rights Reserved.
 * The contents of this file constitute Original Code as defined in and are 
 * subject to the Apple Public Source License Version 1.1 (the "License").  
 * You may not use this file except in compliance with the License.  Please 
 * obtain a copy of the License at http://www.apple.com/publicsource and 
 * read it before using this file.
 * 
 * This Original Code and all software distributed under the License are 
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, 
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS 
 * FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the License for 
 * the specific language governing rights and limitations under the 
 * License.
 * 
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
/*
	File:		QTSS_Private.h

	Contains:	Implementation-specific structures and typedefs used by the
				implementation of QTSS API in the Darwin Streaming Server
					
	$Log: QTSS_Private.h,v $
	Revision 1.2  2000/01/27 02:09:04  rtucker
	Merge changes from v81
	
	Revision 1.6  2000/01/26 21:34:48  serenyi
	Added QTSS_RequestIdleTime callback
	
	Revision 1.5  1999/12/21 01:43:30  lecroy
	*** empty log message ***
	
	Revision 1.4  1999/12/14 21:31:00  lecroy
	various QT -> Darwin name changes, fixed up Installit
	
	Revision 1.3  1999/10/07 09:02:24  serenyi
	Changed bool to Bool16, changed QTSS_Dictionary to QTSS_Object
	
	Revision 1.2  1999/09/24 19:52:57  murata
	Added new dictionary fields for accessing Client Environment info. Also added new
	API function for converting qtss milliseconds to Unix seconds (1970 based).
	
	Revision 1.1.1.1  1999/09/22 23:01:15  lecroy
	importing
	
	Revision 1.1.1.1  1999/09/13 16:16:36  rtucker
	initial checkin from rrv14
	
	
	
*/


#ifndef QTSS_PRIVATE_H
#define QTSS_PRIVATE_H

#ifdef __cplusplus
extern "C" {
#endif

#include "OSHeaders.h"
#include "QTSS.h"

class QTSSModule;
class Task;

typedef QTSS_Error 	(*QTSS_CallbackProcPtr)(...);
typedef void*			(*QTSS_CallbackPtrProcPtr)(...);

enum
{
	// Indexes for each callback routine. Addresses of the callback routines get
	// placed in an array.
	
	kNewCallback 					= 0,
	kDeleteCallback					= 1,
	kMillisecondsCallback			= 2,
	kConvertToUnixTimeCallback		= 3,
	kAddRoleCallback 				= 4,
	kAddAttributeCallback 			= 5,
	kIDForTagCallback 				= 6,
	kGetAttributePtrByIDCallback 	= 7,
	kGetAttributeByIDCallback 		= 8,
	kSetAttributeByIDCallback 		= 9,
	kWriteCallback					= 10,
	kWriteVCallback					= 11,
	kFlushCallback					= 12,
	kAddServiceCallback				= 13,
	kIDForServiceCallback			= 14,
	kDoServiceCallback				= 15,
	kSendRTSPHeadersCallback		= 16,
	kAppendRTSPHeadersCallback		= 17,
	kSendStandardRTSPCallback		= 18,
	kAddRTPStreamCallback			= 19,
	kPlayCallback					= 20,
	kPauseCallback					= 21,
	kTeardownCallback				= 22,
	kCreateEventContextCallback		= 23,
	kDestroyEventContextCallback	= 24,
	kRequestEventCallback			= 25,
	kSetIdleTimerCallback			= 26,
	
	kLastCallback 					= 27
};

typedef struct {
	// Callback function pointer array
	QTSS_CallbackProcPtr addr [kLastCallback];
} QTSS_Callbacks, *QTSS_CallbacksPtr;

typedef struct
{
	UInt32					inServerAPIVersion;
	QTSS_CallbacksPtr 		inCallbacks;
	QTSS_StreamRef	 		inErrorLogStream;
	UInt32					outStubLibraryVersion;
	QTSS_DispatchFuncPtr	outDispatchFunction;
	
} QTSS_PrivateArgs, *QTSS_PrivateArgsPtr;

typedef struct
{
	QTSSModule*	curModule;	// this structure is setup in each thread
	QTSS_Role	curRole;	// before invoking a module in a role. Sometimes
	Task*		curTask;	// this info. helps callback implementation
	Bool16		eventRequested;
	SInt64		idleTime;	// If a module has requested idle time.
	
} QTSS_ModuleState, *QTSS_ModuleStatePtr;

QTSS_StreamRef	GetErrorLogStream();


#ifdef __cplusplus
}
#endif

#endif
