/*
 * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * Copyright (c) 1999 Apple Computer, Inc.  All Rights Reserved.
 * The contents of this file constitute Original Code as defined in and are 
 * subject to the Apple Public Source License Version 1.1 (the "License").  
 * You may not use this file except in compliance with the License.  Please 
 * obtain a copy of the License at http://www.apple.com/publicsource and 
 * read it before using this file.
 * 
 * This Original Code and all software distributed under the License are 
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, 
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS 
 * FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the License for 
 * the specific language governing rights and limitations under the 
 * License.
 * 
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
/*
	File:		OS.h

	Contains:  	OS utility functions. Memory allocation, time, etc.

	$Log: OS.h,v $
	Revision 1.1.1.1  2000/01/17 23:35:57  wsanchez
	Import of StreamingServer-2.0 (Apple)
	
	Revision 1.5  1999/12/21 01:43:31  lecroy
	*** empty log message ***
	
	Revision 1.4  1999/11/22 21:56:14  serenyi
	Turns out the GetUInt16FromMemory is not necessary after all
	
	Revision 1.3  1999/11/22 01:29:00  serenyi
	Added some inline functions for memory copying which work on platforms that require longword aligned access
	
	Revision 1.2  1999/09/24 19:59:46  murata
	Added new time conversion support.
	
	Revision 1.1.1.1  1999/09/22 23:01:18  lecroy
	importing
	
	Revision 1.1.1.1  1999/09/13 16:16:37  rtucker
	initial checkin from rrv14
	
	
	Revision 1.4  1999/02/26 04:55:29  cl
	Fixed ObjectDeleter reversed assert conditional
	
	Revision 1.3  1999/02/25 06:41:06  cl
	Removed "assert" call and replaced with inline code due to include files recursion
	
	Revision 1.2  1999/02/19 23:05:31  ds
	Created
		

*/

#ifndef _OS_H_
#define _OS_H_

#include "OSHeaders.h"
#include <string.h>

class OS
{
	public:
	
		//call this before calling anything else
		static void	Initialize();

		static SInt32 Min(SInt32 a, SInt32 b) 	{ if (a < b) return a; return b; }
		
		static SInt64 	Milliseconds();
		static SInt64	Microseconds();
		
		// Some processors (MIPS, Sparc) cannot handle non word aligned memory
		// accesses. So, we need to provide functions to safely get at non-word
		// aligned memory.
		static inline UInt32	GetUInt32FromMemory(UInt32* inP);

		//because the OS doesn't seem to have this function
		static SInt64	HostToNetworkSInt64(SInt64 hostOrdered);
		
		static SInt64	ConvertToSecondsSince1900(SInt64 inMilliseconds)
							{ return ConvertMsecToFixed64Sec(sMsecSince1900 + inMilliseconds); }
		static SInt64	ConvertMsecToFixed64Sec(SInt64 inMilliseconds)
							{ return (SInt64)(.5 + inMilliseconds * 4294967.296L); }

		static SInt64	ConvertToMsecSince1970(SInt64 inMilliseconds)
							{ return sMsecSince1970 + inMilliseconds; }
		
		static time_t	ConvertToUnixTime(SInt64 inMilliseconds)
							{ return (time_t)  (ConvertToMsecSince1970(inMilliseconds) / (SInt64) 1000); }

		static time_t 	UnixTime(void) // Seconds since 1970
							{ return ConvertToUnixTime(Milliseconds()); }
							
		//Both these functions return QTSS_NoErr, QTSS_FileExists, or POSIX errorcode
		//Makes whatever directories in this path that don't exist yet 
		static OS_Error RecursiveMakeDir(char *inPath, mode_t mode);
		//Makes the directory at the end of this path
		static OS_Error MakeDir(char *inPath, mode_t mode);
		
	private:
	
		static double sDivisor;
		static double sMicroDivisor;
		static SInt64 sMsecSince1900;
		static SInt64 sMsecSince1970;
		static SInt64 s1970Msec;
		static SInt64 sInitialMsec;
		static SInt32 sMemoryErr;
		static void SetDivisor();
};

inline UInt32	OS::GetUInt32FromMemory(UInt32* inP)
{
#if ALLOW_NON_WORD_ALIGN_ACCESS
	return *inP;
#else
	UInt32 temp = 0;
	::memcpy(&temp, inP, sizeof(UInt32));
	return temp;
#endif
}


#endif
