/*
 * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * Copyright (c) 1999 Apple Computer, Inc.  All Rights Reserved.
 * The contents of this file constitute Original Code as defined in and are 
 * subject to the Apple Public Source License Version 1.1 (the "License").  
 * You may not use this file except in compliance with the License.  Please 
 * obtain a copy of the License at http://www.apple.com/publicsource and 
 * read it before using this file.
 * 
 * This Original Code and all software distributed under the License are 
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, 
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS 
 * FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the License for 
 * the specific language governing rights and limitations under the 
 * License.
 * 
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
/*
	File:		OSArrayObjectDeleter.h

	Contains:  	Auto object for deleting arrays.

	$Log: OSArrayObjectDeleter.h,v $
	Revision 1.1.1.1  2000/01/17 23:35:57  wsanchez
	Import of StreamingServer-2.0 (Apple)
	
	Revision 1.3  1999/12/21 01:43:32  lecroy
	*** empty log message ***
	
	Revision 1.2  1999/10/16 00:32:57  murata
	Fix the template for SetVal to return void
	
	Revision 1.1.1.1  1999/09/22 23:01:18  lecroy
	importing
	
	Revision 1.1.1.1  1999/09/13 16:16:37  rtucker
	initial checkin from rrv14
	
	
	
	Revision 1.2  1999/02/19 23:05:31  ds
	Created
		

*/

#ifndef __OS_ARRAY_OBJECT_DELETER_H__
#define __OS_ARRAY_OBJECT_DELETER_H__

#include "MyAssert.h"

template <class T>
class OSArrayObjectDeleter
{
	public:
		OSArrayObjectDeleter(T* victim) : fT(victim)  {}
		~OSArrayObjectDeleter() { delete [] fT; }
		
		void SetObject(T* victim) 
		{
			Assert(fT == NULL);
			fT = victim; 
		}
		T* GetObject() { return fT; }
		
		operator T*() { return fT; }
	
	private:
	
		T* fT;
};

typedef OSArrayObjectDeleter<char> OSCharArrayDeleter;

#endif //__OS_OBJECT_ARRAY_DELETER_H__
