/*
 * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * Copyright (c) 1999 Apple Computer, Inc.  All Rights Reserved.
 * The contents of this file constitute Original Code as defined in and are 
 * subject to the Apple Public Source License Version 1.1 (the "License").  
 * You may not use this file except in compliance with the License.  Please 
 * obtain a copy of the License at http://www.apple.com/publicsource and 
 * read it before using this file.
 * 
 * This Original Code and all software distributed under the License are 
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, 
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS 
 * FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the License for 
 * the specific language governing rights and limitations under the 
 * License.
 * 
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
/*
	File:		OSCond.h

	Contains:	A simple condition variable abstraction
	
	$Log: OSCond.h,v $
	Revision 1.1.1.1  2000/01/17 23:35:57  wsanchez
	Import of StreamingServer-2.0 (Apple)
	
	Revision 1.3  1999/12/21 01:43:32  lecroy
	*** empty log message ***
	
	Revision 1.2  1999/11/22 01:28:05  serenyi
	Implemented recursive mutexes for platforms (IRIX, Solaris) that don't support mutex recursion internally
	
	Revision 1.1.1.1  1999/09/22 23:01:18  lecroy
	importing
	
	Revision 1.1.1.1  1999/09/13 16:16:38  rtucker
	initial checkin from rrv14
	
	
	Revision 1.2  1999/02/19 23:06:12  ds
	Created
		

*/

#ifndef _OSCOND_H_
#define _OSCOND_H_

#if !__PTHREADS__
#include "mycondition.h"
#else
#include <pthread.h>
#endif

#include "OSMutex.h"
#include "MyAssert.h"

class OSCond 
{
	public:

		OSCond();
		~OSCond();
		
		inline void		Broadcast();
		inline void 	Signal();
		inline void 	Wait(OSMutex* inMutex, SInt32 inTimeoutInMilSecs = 0);

	private:

#if !__PTHREADS__
		mycondition_t 		fCondition;
#else
		pthread_cond_t		fCondition;
		void				TimedWait(OSMutex* inMutex, SInt32 inTimeoutInMilSecs);
#endif
};

inline void OSCond::Wait(OSMutex* inMutex, SInt32 inTimeoutInMilSecs)
{ 
#if !__PTHREADS__
	Assert(fCondition != NULL);
	mycondition_wait(fCondition, inMutex->fMutex, inTimeoutInMilSecs);
#else
	this->TimedWait(inMutex, inTimeoutInMilSecs);
#endif
}

inline void OSCond::Signal()
{
#if !__PTHREADS__
	Assert(fCondition != NULL)
	mycondition_signal(fCondition);
#else
	pthread_cond_signal(&fCondition);
#endif
}

inline void OSCond::Broadcast()
{
#if !__PTHREADS__
	Assert(fCondition != NULL);
	mycondition_broadcast(fCondition);
#else
	pthread_cond_broadcast(&fCondition);
#endif
}


#endif //_OSCOND_H_
