/*
 * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * Copyright (c) 1999 Apple Computer, Inc.  All Rights Reserved.
 * The contents of this file constitute Original Code as defined in and are 
 * subject to the Apple Public Source License Version 1.1 (the "License").  
 * You may not use this file except in compliance with the License.  Please 
 * obtain a copy of the License at http://www.apple.com/publicsource and 
 * read it before using this file.
 * 
 * This Original Code and all software distributed under the License are 
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, 
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS 
 * FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the License for 
 * the specific language governing rights and limitations under the 
 * License.
 * 
 * 
 * @APPLE_LICENSE_HEADER_END@
 */

#ifndef OSHeaders_H
#define OSHeaders_H

/* Platform-specific components */
#if __linux__ || __linuxppc__ || __FreeBSD__ || __MacOSX__
	
        /* Defines */
        #define _64BITARG_ "q"

	/* Includes */
	#include <sys/types.h>
	
	/* Constants */
	#define QT_TIME_TO_LOCAL_TIME	(-2082844800)
	#define QT_PATH_SEPARATOR		'/'

	/* Typedefs */
	typedef unsigned char		UInt8;
	typedef signed char			SInt8;
	typedef unsigned short		UInt16;
	typedef signed short		SInt16;
	typedef unsigned long		UInt32;
	typedef signed long			SInt32;
	typedef signed long long	SInt64;
	typedef unsigned long long	UInt64;
	typedef float				Float32;
	typedef double				Float64;
	typedef UInt16				Bool16;
	typedef UInt8				Bool8;
	
	typedef unsigned long		FourCharCode;
	typedef FourCharCode		OSType;

	#ifdef	FOUR_CHARS_TO_INT
	#error Conflicting Macro "FOUR_CHARS_TO_INT"
	#endif

	#define	FOUR_CHARS_TO_INT( c1, c2, c3, c4 )  ( c1 << 24 | c2 << 16 | c3 << 8 | c4 )

	#ifdef	TW0_CHARS_TO_INT
	#error Conflicting Macro "TW0_CHARS_TO_INT"
	#endif
		
	#define	TW0_CHARS_TO_INT( c1, c2 )  ( c1 << 8 | c2 )

	//#define thread_t	pthread_t
	//#define cthread_errno() errno

#elif __MACOS__

	#define _64BITARG_ "ll"
	
	/* Includes */
	#include <sys/types.h>

	/* Constants */
	#define QT_TIME_TO_LOCAL_TIME	(126115200)
	#define QT_PATH_SEPARATOR		':'
	
	/* Typedefs */
	typedef unsigned char		UInt8;
	typedef signed char			SInt8;
	typedef unsigned short		UInt16;
	typedef signed short		SInt16;
	typedef unsigned long		UInt32;
	typedef signed long			SInt32;
	typedef signed long long	SInt64;
	typedef unsigned long long	UInt64;
	typedef float				Float32;
	typedef short double		Float64;
	typedef UInt16				Bool16;
	typedef UInt8				Bool8;
	
	typedef unsigned long		FourCharCode;
	typedef FourCharCode		OSType;

	#ifdef	FOUR_CHARS_TO_INT
	#error Conflicting Macro "FOUR_CHARS_TO_INT"
	#endif

	#define	FOUR_CHARS_TO_INT( c1, c2, c3, c4 )  ( c1 << 24 | c2 << 16 | c3 << 8 | c4 )

	#ifdef	TW0_CHARS_TO_INT
	#error Conflicting Macro "TW0_CHARS_TO_INT"
	#endif
		
	#define	TW0_CHARS_TO_INT( c1, c2 )  ( c1 << 8 | c2 )

#elif __sgi
	/* Defines */
	#define _64BITARG_ "ll"

	/* Includes */
	#include <sys/types.h>
	#include <netinet/in.h>
	#include <pthread.h>
	
	/* Constants */
	#define QT_TIME_TO_LOCAL_TIME	(-2082844800)
	#define QT_PATH_SEPARATOR		'/'

	/* Typedefs */
	typedef unsigned char		boolean;
	#define true				1
	#define false				0

	typedef unsigned char		UInt8;
	typedef signed char			SInt8;
	typedef unsigned short		UInt16;
	typedef signed short		SInt16;
	typedef unsigned long		UInt32;
	typedef signed long			SInt32;
	typedef signed long long	SInt64;
	typedef unsigned long long	UInt64;
	typedef float				Float32;
	typedef double				Float64;
	
	typedef unsigned long		FourCharCode;
	typedef FourCharCode		OSType;

	/* Nulled-out new() for use without memory debugging */
	#define NEW(t,c,v) new c v
	#define NEW_ARRAY(t,c,n) new c[n]

	#define thread_t	pthread_t
	#define cthread_errno() errno

	#ifdef	FOUR_CHARS_TO_INT
	#error Conflicting Macro "FOUR_CHARS_TO_INT"
	#endif

	#define	FOUR_CHARS_TO_INT( c1, c2, c3, c4 )  ( c1 << 24 | c2 << 16 | c3 << 8 | c4 )

	#ifdef	TW0_CHARS_TO_INT
	#error Conflicting Macro "TW0_CHARS_TO_INT"
	#endif
		
	#define	TW0_CHARS_TO_INT( c1, c2 )  ( c1 << 8 | c2 )

#elif defined(sun) // && defined(sparc)

	/* Defines */
	#define _64BITARG_ "ll"

	/* Includes */
	#include <sys/types.h>
	#include <sys/byteorder.h>
	
	/* Constants */
	#define QT_TIME_TO_LOCAL_TIME	(-2082844800)
	#define QT_PATH_SEPARATOR		'/'

	/* Typedefs */
	//typedef unsigned char		Bool16;
	//#define true				1
	//#define false				0

	typedef unsigned char		UInt8;
	typedef signed char			SInt8;
	typedef unsigned short		UInt16;
	typedef signed short		SInt16;
	typedef unsigned long		UInt32;
	typedef signed long			SInt32;
	typedef signed long long	SInt64;
	typedef unsigned long long	UInt64;
	typedef float				Float32;
	typedef double				Float64;
	typedef UInt16				Bool16;
	typedef UInt8				Bool8;
	
	typedef unsigned long		FourCharCode;
	typedef FourCharCode		OSType;

	#ifdef	FOUR_CHARS_TO_INT
	#error Conflicting Macro "FOUR_CHARS_TO_INT"
	#endif

	#define	FOUR_CHARS_TO_INT( c1, c2, c3, c4 )  ( c1 << 24 | c2 << 16 | c3 << 8 | c4 )

	#ifdef	TW0_CHARS_TO_INT
	#error Conflicting Macro "TW0_CHARS_TO_INT"
	#endif
		
	#define	TW0_CHARS_TO_INT( c1, c2 )  ( c1 << 8 | c2 )
	
	//#define thread_t	pthread_t
#endif

enum
{
	OS_NoErr = 0,
	OS_BadURLFormat = -100
};
typedef UInt32 OS_Error;


#endif /* OSHeaders_H */
