/*
 * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * Copyright (c) 1999 Apple Computer, Inc.  All Rights Reserved.
 * The contents of this file constitute Original Code as defined in and are 
 * subject to the Apple Public Source License Version 1.1 (the "License").  
 * You may not use this file except in compliance with the License.  Please 
 * obtain a copy of the License at http://www.apple.com/publicsource and 
 * read it before using this file.
 * 
 * This Original Code and all software distributed under the License are 
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, 
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS 
 * FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the License for 
 * the specific language governing rights and limitations under the 
 * License.
 * 
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
/*
	File:		ResizeableStringFormatter.cpp

	Contains:	Implements object defined in ResizeableStringFormatter.h
	
	$Log: ResizeableStringFormatter.cpp,v $
	Revision 1.2  2000/01/27 02:09:05  rtucker
	Merge changes from v81
	
	Revision 1.3  2000/01/19 19:38:22  serenyi
	Moved to new NEW macro
	
	Revision 1.2  1999/12/21 01:43:35  lecroy
	*** empty log message ***
	
	Revision 1.1.1.1  1999/09/22 23:01:20  lecroy
	importing
	
	Revision 1.1  1999/09/15 17:03:48  serenyi
	Created
	
	Revision 1.2  1999/02/19 23:08:33  ds
	Created
	

*/

#include "ResizeableStringFormatter.h"
#include "OSMemory.h"

void	ResizeableStringFormatter::BufferIsFull(char* inBuffer, UInt32 inBufferLen)
{
	//allocate a buffer twice as big as the old one, and copy over the contents
	UInt32 theNewBufferSize = this->GetTotalBufferSize() * 2;
	char* theNewBuffer = NEW char[theNewBufferSize];
	::memcpy(theNewBuffer, inBuffer, inBufferLen);

	//if the old buffer was dynamically allocated also, we'd better delete it.
	if (inBuffer != fOriginalBuffer)
		delete [] inBuffer;
	
	fStartPut = theNewBuffer;
	fCurrentPut = theNewBuffer + inBufferLen;
	fEndPut = theNewBuffer + theNewBufferSize;
}
