/*
 * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * Copyright (c) 1999 Apple Computer, Inc.  All Rights Reserved.
 * The contents of this file constitute Original Code as defined in and are 
 * subject to the Apple Public Source License Version 1.1 (the "License").  
 * You may not use this file except in compliance with the License.  Please 
 * obtain a copy of the License at http://www.apple.com/publicsource and 
 * read it before using this file.
 * 
 * This Original Code and all software distributed under the License are 
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, 
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS 
 * FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the License for 
 * the specific language governing rights and limitations under the 
 * License.
 * 
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
/*
	File:		SocketUtils.h

	Contains:	Some static routines for dealing with networking
					
	$Log: SocketUtils.h,v $
	Revision 1.1.1.1  2000/01/17 23:35:58  wsanchez
	Import of StreamingServer-2.0 (Apple)
	
	Revision 1.4  1999/12/21 01:43:35  lecroy
	*** empty log message ***
	
	Revision 1.3  1999/10/27 17:53:46  serenyi
	Integrated Matthias's MacOS / GUSI changes
	
	Revision 1.2  1999/10/07 08:55:02  serenyi
	Changed bool to Bool16, changed QTSS_Dictionary to QTSS_Object
	
	Revision 1.1.1.1  1999/09/22 23:01:21  lecroy
	importing
	
	Revision 1.1.1.1  1999/09/13 16:16:39  rtucker
	initial checkin from rrv14
	
	
	Revision 1.2  1999/02/19 23:13:19  ds
	Created
	
	
*/

#ifndef __SOCKETUTILS_H__
#define __SOCKETUTILS_H__

#ifdef PRAGMA_ONCE
#pragma once
#endif

#include <sys/socket.h>
#include <netinet/in.h>
#include <net/if.h>
#include "ev.h"

#include "OSHeaders.h"
#include "MyAssert.h"
#include "StrPtrLen.h"

class SocketUtils
{
	public:

		//call initialize before using any socket functions
		static void Initialize();
		
		//static utility routines
		static Bool16 	IsMulticastIPAddr(UInt32 inAddress);
		static Bool16	IsLocalIPAddr(UInt32 inAddress);

		//This function converts an integer IP address to a dotted-decimal string.
		//This function is NOT THREAD SAFE!!!
		static void ConvertAddrToString(const struct in_addr& theAddr, StrPtrLen* outAddr);
		
		//You can get at all the IP addrs and DNS names on this machine this way
		static UInt32		GetNumIPAddrs() { return sNumIPAddrs; }
		static inline UInt32		GetIPAddr(UInt32 inAddrIndex);
		static inline StrPtrLen*	GetIPAddrStr(UInt32 inAddrIndex);
		static inline StrPtrLen*	GetDNSNameStr(UInt32 inDNSIndex);
			
	private:

		//Utility function used by Initialize
		static Bool16 IncrementIfReqIter(char** inIfReqIter, ifreq* ifr);

		//For storing relevent information about each IP interface
		struct IPAddrInfo
		{
			UInt32 		fIPAddr;
			StrPtrLen	fIPAddrStr;
			StrPtrLen	fDNSNameStr;
		};
		
		static IPAddrInfo*				sIPAddrInfoArray;
		static UInt32					sNumIPAddrs;
};

inline UInt32	SocketUtils::GetIPAddr(UInt32 inAddrIndex)
{
	Assert(sIPAddrInfoArray != NULL);
	Assert(inAddrIndex < sNumIPAddrs);
	return sIPAddrInfoArray[inAddrIndex].fIPAddr;
}

inline StrPtrLen*	SocketUtils::GetIPAddrStr(UInt32 inAddrIndex)
{
	Assert(sIPAddrInfoArray != NULL);
	Assert(inAddrIndex < sNumIPAddrs);
	return &sIPAddrInfoArray[inAddrIndex].fIPAddrStr;
}

inline StrPtrLen*	SocketUtils::GetDNSNameStr(UInt32 inDNSIndex)
{
	Assert(sIPAddrInfoArray != NULL);
	Assert(inDNSIndex < sNumIPAddrs);
	return &sIPAddrInfoArray[inDNSIndex].fDNSNameStr;
}

#endif // __SOCKETUTILS_H__

