/*
 * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * Copyright (c) 1999 Apple Computer, Inc.  All Rights Reserved.
 * The contents of this file constitute Original Code as defined in and are 
 * subject to the Apple Public Source License Version 1.1 (the "License").  
 * You may not use this file except in compliance with the License.  Please 
 * obtain a copy of the License at http://www.apple.com/publicsource and 
 * read it before using this file.
 * 
 * This Original Code and all software distributed under the License are 
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, 
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS 
 * FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the License for 
 * the specific language governing rights and limitations under the 
 * License.
 * 
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
/*
	File:		TCPListenerSocket.h

	Contains:	A TCP listener socket. When a new connection comes in, the listener
				attempts to assign the new connection to a socket object and a Task
				object. Derived classes must implement a method of getting new
				Task & socket objects
					
	$Log: TCPListenerSocket.h,v $
	Revision 1.1.1.1  2000/01/17 23:35:58  wsanchez
	Import of StreamingServer-2.0 (Apple)
	
	Revision 1.4  1999/12/21 01:43:37  lecroy
	*** empty log message ***
	
	Revision 1.3  1999/11/08 22:53:19  serenyi
	Updated code to reflect latest version of QTSS API
	
	Revision 1.2  1999/10/07 08:55:02  serenyi
	Changed bool to Bool16, changed QTSS_Dictionary to QTSS_Object
	
	Revision 1.1.1.1  1999/09/22 23:01:22  lecroy
	importing
	
	Revision 1.1.1.1  1999/09/13 16:16:39  rtucker
	initial checkin from rrv14
	
	
	Revision 1.2  1999/02/19 23:13:21  ds
	Created
	
	
*/


#ifndef __TCPLISTENERSOCKET_H__
#define __TCPLISTENERSOCKET_H__

#include "TCPSocket.h"
#include "IdleTask.h"

class TCPListenerSocket : public TCPSocket, public IdleTask
{
	public:

		TCPListenerSocket() : 	TCPSocket(NULL, Socket::kNonBlockingSocketType), IdleTask(),
								fAddr(0), fPort(0), fOutOfDescriptors(false) {}
		virtual ~TCPListenerSocket() {}
		
		//addr = listening address. port = listening port. Automatically
		//starts listening
		OS_Error		Initialize(UInt32 addr, UInt16 port);

		//You can query the listener to see if it is failing to accept
		//connections because the OS is out of descriptors.
		Bool16		IsOutOfDescriptors() { return fOutOfDescriptors; }

		//derived object must implement a way of getting tasks & sockets to this object 
		virtual Task*	GetSessionTask(TCPSocket** outSocket) = 0;
		
		virtual SInt64	Run();
			
	private:
	
		enum
		{
			kTimeBetweenAcceptsInMsec = 1000,	//UInt32
			kListenQueueLength = 128			//UInt32
		};

		virtual void ProcessEvent(int eventBits);
		OS_Error 	Listen(UInt32 queueLength);

		UInt32			fAddr;
		UInt16			fPort;
		
		Bool16			fOutOfDescriptors;
};
#endif // __TCPLISTENERSOCKET_H__

