/*
 * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * Copyright (c) 1999 Apple Computer, Inc.  All Rights Reserved.
 * The contents of this file constitute Original Code as defined in and are 
 * subject to the Apple Public Source License Version 1.1 (the "License").  
 * You may not use this file except in compliance with the License.  Please 
 * obtain a copy of the License at http://www.apple.com/publicsource and 
 * read it before using this file.
 * 
 * This Original Code and all software distributed under the License are 
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, 
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS 
 * FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the License for 
 * the specific language governing rights and limitations under the 
 * License.
 * 
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
/*
	File:		TCPSocket.cpp

	Contains:	implements TCPSocket class
					
	
	$Log: TCPSocket.cpp,v $
	Revision 1.3  2000/03/09 22:52:17  serenyi
	Made work on freebsd
	
	Revision 1.2  2000/01/27 02:09:05  rtucker
	Merge changes from v81
	
	Revision 1.5  2000/01/19 19:38:22  serenyi
	Moved to new NEW macro
	
	Revision 1.4  1999/12/21 01:43:37  lecroy
	*** empty log message ***
	
	Revision 1.3  1999/10/27 17:53:47  serenyi
	Integrated Matthias's MacOS / GUSI changes
	
	Revision 1.2  1999/09/30 00:24:40  serenyi
	Snarf bug fixes?
	
	Revision 1.1.1.1  1999/09/22 23:01:22  lecroy
	importing
	
	Revision 1.1.1.1  1999/09/13 16:16:39  rtucker
	initial checkin from rrv14
	
	
	Revision 1.2  1999/02/19 23:13:22  ds
	Created
	
	
*/

#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <sys/errno.h>

#include "TCPSocket.h"
#include "SocketUtils.h"
#include "OS.h"


void TCPSocket::SnarfSocket( TCPSocket & fromSocket )
{
	// take the connection away from the other socket
	// and use it as our own.
	Assert(fFileDesc == EventContext::kInvalidFileDesc);
	this->Set( fromSocket.fFileDesc, &fromSocket.fRemoteAddr, fromSocket.fListener );
	
	// clear the old socket so he doesn't close and the like
	struct	sockaddr_in	 remoteaddr;
	
	::memset( &remoteaddr, 0, sizeof( remoteaddr ) );

	fromSocket.Set( EventContext::kInvalidFileDesc, &remoteaddr, NULL );

	// get the event context too
	this->SnarfEventContext( fromSocket );

}

void TCPSocket::Set(int inSocket, struct sockaddr_in* remoteaddr, TCPSocket *listener)
{
	fRemoteAddr = *remoteaddr;
	fListener = listener;
	fFileDesc = inSocket;
	
	if ( inSocket != EventContext::kInvalidFileDesc ) 
	{
		//make sure to find out what IP address this connection is actually occuring on. That
		//way, we can report correct information to clients asking what the connection's IP is
#if __MacOSX__ || __FreeBSD__
		int len = sizeof(fLocalAddr);
#else
		socklen_t len = sizeof(fLocalAddr);
#endif
		int err = ::getsockname(fFileDesc, (struct sockaddr*)&fLocalAddr, &len);
		AssertV(err == 0, OSThread::GetErrno());
		fState |= kBound;
		fState |= kConnected;
	}
	else
		fState = 0;
}

StrPtrLen*	TCPSocket::GetRemoteAddrStr()
{
	if (fRemoteStr.Len == kIPAddrBufSize)
		SocketUtils::ConvertAddrToString(fRemoteAddr.sin_addr, &fRemoteStr);
	return &fRemoteStr;
}

OS_Error  TCPSocket::Connect(UInt32 inRemoteAddr, UInt16 inRemotePort)
{
	::memset(&fRemoteAddr, 0, sizeof(fRemoteAddr));
    fRemoteAddr.sin_family = AF_INET;        /* host byte order */
    fRemoteAddr.sin_port = htons(inRemotePort); /* short, network byte order */
    fRemoteAddr.sin_addr.s_addr = htonl(inRemoteAddr);

    /* don't forget to error check the connect()! */
    int err = ::connect(fFileDesc, (sockaddr *)&fRemoteAddr, sizeof(fRemoteAddr));
	fState |= kConnected;
	
	if (err == -1)
	{
		fRemoteAddr.sin_port = 0;
		fRemoteAddr.sin_addr.s_addr = 0;
		return (OS_Error)OSThread::GetErrno();
	}
	
	return OS_NoErr;

}

#if 0
OS_Error	TCPSocket::CheckAsyncConnect()
{
	// If the socket has gone readable, a connection failure has occurred
	if (fEventBits & EV_RE)
		return ENOTCONN;
	// If the socket has gone writeable, we are connected.
	else if (fEventBits & EV_WR)
	{
		fState |= kConnected;
		return OS_NoErr;
	}
	// No event has transpired, so our connect is still in progress
	return EINPROGRESS;
}
#endif
