/*
 * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * Copyright (c) 1999 Apple Computer, Inc.  All Rights Reserved.
 * The contents of this file constitute Original Code as defined in and are 
 * subject to the Apple Public Source License Version 1.1 (the "License").  
 * You may not use this file except in compliance with the License.  Please 
 * obtain a copy of the License at http://www.apple.com/publicsource and 
 * read it before using this file.
 * 
 * This Original Code and all software distributed under the License are 
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, 
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS 
 * FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the License for 
 * the specific language governing rights and limitations under the 
 * License.
 * 
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
/*
	File:		TCPSocket.h

	Contains:	TCP socket object
					
	
	$Log: TCPSocket.h,v $
	Revision 1.1.1.1  2000/01/17 23:35:58  wsanchez
	Import of StreamingServer-2.0 (Apple)
	
	Revision 1.4  1999/12/21 01:43:37  lecroy
	*** empty log message ***
	
	Revision 1.3  1999/10/27 17:53:47  serenyi
	Integrated Matthias's MacOS / GUSI changes
	
	Revision 1.2  1999/09/30 00:24:40  serenyi
	Snarf bug fixes?
	
	Revision 1.1.1.1  1999/09/22 23:01:22  lecroy
	importing
	
	Revision 1.1.1.1  1999/09/13 16:16:39  rtucker
	initial checkin from rrv14
	
	
	Revision 1.2  1999/02/19 23:13:22  ds
	Created
	
	
*/

#ifndef __TCPSOCKET_H__
#define __TCPSOCKET_H__

#include <stdio.h>
#include <stdlib.h>

#include <sys/types.h>
#include <sys/socket.h>

#include "Socket.h"
#include "Task.h"
#include "StrPtrLen.h"

class TCPListenerSocket;

class TCPSocket : public Socket
{
	public:

		//TCPSocket takes an optional task object which will get notified when
		//certain events happen on this socket. Those events are:
		//
		//S_DATA:				Data is currently available on the socket.
		//S_CONNECTIONCLOSING:	Client is closing the connection. No longer necessary
		//						to call Close or Disconnect, Snd & Rcv will fail.
		TCPSocket(Task *notifytask, UInt32 inSocketType)
			: 	Socket(notifytask, inSocketType), fListener(NULL), 
				fRemoteStr(fRemoteBuffer, kIPAddrBufSize)  {}
		virtual ~TCPSocket() {}

		//Open
		OS_Error 	Open() { return Socket::Open(SOCK_STREAM); }

		// Connect. Attempts to connect to the specified remote host. If this
		// is a non-blocking socket, this function may return EINPROGRESS, in which
		// case caller must wait for either an EV_RE or an EV_WR. You may call
		// CheckAsyncConnect at any time, which will return OS_NoErr if the connect
		// has completed, EINPROGRESS if it is still in progress, or an appropriate error
		// if the connect failed.
		OS_Error	Connect(UInt32 inRemoteAddr, UInt16 inRemotePort);
		//OS_Error	CheckAsyncConnect();

		// Basically a copy constructor for this object, also NULLs out the data
		// in tcpSocket.		
		void		SnarfSocket( TCPSocket & tcpSocket );

		//ACCESSORS:
		//Returns NULL if not currently available.
		
		UInt32		GetRemoteAddr() { return ntohl(fRemoteAddr.sin_addr.s_addr); }
		UInt16		GetRemotePort() { return ntohs(fRemoteAddr.sin_port); }
		//This function is NOT thread safe!
		StrPtrLen*	GetRemoteAddrStr();
		TCPSocket*	GetListener()			{ return fListener; }

	protected:

		void 		Set(int inSocket, struct sockaddr_in* remoteaddr, TCPSocket *listener);
							
		enum
		{
			kIPAddrBufSize = 20	//UInt32
		};

		TCPSocket*			fListener;
		struct sockaddr_in 	fRemoteAddr;
		char fRemoteBuffer[kIPAddrBufSize];
		StrPtrLen fRemoteStr;

		
		friend class TCPListenerSocket;
};
#endif // __TCPSOCKET_H__

