/*
 * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * Copyright (c) 1999 Apple Computer, Inc.  All Rights Reserved.
 * The contents of this file constitute Original Code as defined in and are 
 * subject to the Apple Public Source License Version 1.1 (the "License").  
 * You may not use this file except in compliance with the License.  Please 
 * obtain a copy of the License at http://www.apple.com/publicsource and 
 * read it before using this file.
 * 
 * This Original Code and all software distributed under the License are 
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, 
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS 
 * FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the License for 
 * the specific language governing rights and limitations under the 
 * License.
 * 
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
/*
	File:		UDPDemuxer.cpp

	Contains:	Implements objects defined in UDPDemuxer.h

	$Log: UDPDemuxer.cpp,v $
	Revision 1.1.1.1  2000/01/17 23:35:58  wsanchez
	Import of StreamingServer-2.0 (Apple)
	
	Revision 1.3  1999/12/21 01:43:38  lecroy
	*** empty log message ***
	
	Revision 1.2  1999/10/27 17:53:47  serenyi
	Integrated Matthias's MacOS / GUSI changes
	
	Revision 1.1.1.1  1999/09/22 23:01:22  lecroy
	importing
	
	Revision 1.1.1.1  1999/09/13 16:16:39  rtucker
	initial checkin from rrv14
	
	
	Revision 1.2  1999/02/19 23:14:15  ds
	Created
	

*/

#include "UDPDemuxer.h"

#include <sys/errno.h>


OS_Error UDPDemuxer::RegisterTask(UInt32 inRemoteAddr, UInt16 inRemotePort,
										UDPDemuxerTask *inTaskP)
{
	Assert(NULL != inTaskP);
	OSMutexLocker locker(&fMutex);
	if (this->GetTask(inRemoteAddr, inRemotePort) != NULL)
		return EPERM;
	inTaskP->Set(inRemoteAddr, inRemotePort);
	fHashTable.Add(inTaskP);
	return OS_NoErr;
}

OS_Error UDPDemuxer::UnregisterTask(UInt32 inRemoteAddr, UInt16 inRemotePort,
											UDPDemuxerTask *inTaskP)
{
	OSMutexLocker locker(&fMutex);
	//remove by executing a lookup based on key information
	UDPDemuxerTask* theTask = this->GetTask(inRemoteAddr, inRemotePort);

	if ((NULL != theTask) && (theTask == inTaskP))
	{
		fHashTable.Remove(theTask);
		return OS_NoErr;
	}
	else
		return EPERM;
}

UDPDemuxerTask* UDPDemuxer::GetTask(UInt32 inRemoteAddr, UInt16 inRemotePort)
{
	UDPDemuxerKey theKey(inRemoteAddr, inRemotePort);
	return fHashTable.Map(&theKey);
}
