/*
 * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * Copyright (c) 1999 Apple Computer, Inc.  All Rights Reserved.
 * The contents of this file constitute Original Code as defined in and are 
 * subject to the Apple Public Source License Version 1.1 (the "License").  
 * You may not use this file except in compliance with the License.  Please 
 * obtain a copy of the License at http://www.apple.com/publicsource and 
 * read it before using this file.
 * 
 * This Original Code and all software distributed under the License are 
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, 
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS 
 * FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the License for 
 * the specific language governing rights and limitations under the 
 * License.
 * 
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
/*
	File:		NetInfoPrefsSource.cpp

	Contains:	Implements object defined in NetInfoPrefsSource.h.

	Written by:	Denis Serenyi

	Copyright:	 1998 by Apple Computer, Inc., all rights reserved.

	Change History (most recent first):

	$Log: NetInfoPrefsSource.cpp,v $
	Revision 1.2  2000/01/27 02:09:08  rtucker
	Merge changes from v81
	
	Revision 1.13  2000/01/26 18:35:15  serenyi
	#2426075. Integrated changes from Kostya Vasilyev
	
	Revision 1.12  2000/01/21 23:52:48  rtucker
	
	logging changes
	
	Revision 1.11  2000/01/19 19:39:38  serenyi
	Moved to new NEW macro
	
	Revision 1.10  1999/12/21 01:43:48  lecroy
	*** empty log message ***
	
	Revision 1.9  1999/12/11 01:52:57  lecroy
	fixed rtsp_timeoutvalue and updated version#
	
	Revision 1.8  1999/11/19 21:28:12  murata
	removed users_movie_folder per bug 2411866
	
	Revision 1.7  1999/11/03 23:58:58  serenyi
	Got rid of disable_timeout_on_play
	
	Revision 1.6  1999/10/29 23:25:17  murata
	Added some QTAccessModule defaults and removed authorization module enables.
	
	Revision 1.5  1999/10/28 20:58:35  serenyi
	Got rid of remote stats prefs
	
	Revision 1.4  1999/10/21 22:36:36  serenyi
	Separated out port pref so we can have multiple defaults, added SetValueByIndex
	
	Revision 1.3  1999/10/19 17:33:29  serenyi
	Added new prefs for the purposes of scaling TCP send buffers, added the ability to set these send buffers
	
	Revision 1.2  1999/09/30 00:27:40  serenyi
	Relaystats shouldn't have a leading slash
	
	Revision 1.1.1.1  1999/09/22 23:01:32  lecroy
	importing
	
	Revision 1.3  1999/09/15 22:28:09  lecroy
	added "file_authorization" config value to enable/disable QTSSAuthorizationModule
	functionality
	
	Revision 1.2  1999/09/15 17:04:10  serenyi
	Added new prefs
	
	Revision 1.1.1.1  1999/09/13 16:16:44  rtucker
	initial checkin from rrv14
	
	
	Revision 1.2  1999/06/08 20:35:34  serenyi
	Allow adding 2 values to one pref, have "rtsp_port" take advantage of this
	
	Revision 1.1  1999/06/01 20:50:03  serenyi
	new files for linux
	

*/

#include "nilib2.h"

#include "NetInfoPrefsSource.h"

static char* gQTSSPropertiesPath = "/services/QuickTimeStreamingServer";

char* kPortProperty[] = {
	// Port pref must be separate because it has multiple values.
	/* 1*/	"rtsp_port", "554", "7070", "8554"
};
const int kNumPorts = 3;

char* kDefaultPrefs[] = {
						/* 1*/	"rtsp_timeout", "0",
						/* 2*/	"maximum_connections", "1000",
						/* 3*/	"movie_folder", "/Local/Library/QuickTimeStreaming/Movies",

						/* 4*/	"request_logging", "disabled",
						/* 5*/	"request_logfile_name", "QTStreamingServer",
						/* 6*/	"request_logfile_dir", "/Local/Library/QuickTimeStreaming/Logs/",
						/* 7*/	"request_logfile_size", "10000000",
						/* 8*/	"request_logfile_interval", "7",

						/* 9*/	"error_logging", "enabled",
						/*10*/	"error_logfile_name", "Error",
						/*11*/	"error_logfile_dir", "/Local/Library/QuickTimeStreaming/Logs/",
						/*12*/	"error_logfile_size", "256000",
						/*13*/	"error_logfile_interval", "7",
						/*14*/	"error_logfile_verbosity", "3",

						/*15*/	"screen_logging", "enabled",
						/*16*/	"break_on_assert", "disabled",
						/*17*/	"bind_ip_addr", "",
						/*18*/	"history_update_interval", "120",
						/*19*/	"maximum_bandwidth", "100000",
						/*20*/	"buffer_seconds", "8",
						/*21*/	"auto_restart", "enabled",
 						/*22*/	"rtp_timeout", "60",

						/*23*/	"total_bytes_update", "1",
						/*24*/	"reflector_delay", "100",
						/*25*/	"reflector_bucket_size", "16",
						/*26*/	"average_bandwidth_update", "60",
						/*27*/	"safe_play_duration", "600",
						/*28*/	"sdp_url", "",
						/*29*/	"admin_email", "",
						/*30*/	"max_advance_send_time", "100",
						/*31*/	"web_stats_url", "",
						/*32*/	"loss_thin_tolerance", "30",
						/*33*/	"num_losses_to_thin", "3",
						/*34*/	"loss_thick_tolerance", "5",
						/*35*/	"num_losses_to_thick", "6",
						/*36*/	"num_worses_to_thin", "2",
						/*37*/	"real_rtsp_timeout", "180",
						/*38*/	"relay_stats_url", "",
						/*39*/	"relay_prefs_file", "/etc/qtssrelay.conf",
						/*40*/	"num_conns_per_ip_addr", "100",
						/*41*/	"do_report_http_connection_ip_address", "disabled",
						/*42*/	"module_folder", "/usr/sbin/QTSSModules",
						/*43*/	"default_authorization_realm", "QT Streaming Server",
						/*44*/	"modAccess_usersfilepath", "/etc/qtusers",
						/*45*/	"modAccess_groupsfilepath", "/etc/qtgroups",
						/*46*/	"modAccess_qtaccessfilename", "qtaccess",
						/*47*/	"run_user_name", "",
						/*48*/	"run_group_name", "",
				
						//!!!be sure to update kDefaultPrefsArraySize below!!!
};
const int kDefaultPrefsArraySize = 48;	//actually 96, but we treat them as key/value paris


NetInfoPrefsSource::NetInfoPrefsSource()
{
	//setup the defaults...
	char tmpValue[1024]= {};
	unsigned int i = 0;
    for (  i = 0; i<kDefaultPrefsArraySize*2; i +=2)
    {
        if (!this->GetValue( kDefaultPrefs[i], tmpValue ))
        {
   			this->SetValue( kDefaultPrefs[i], kDefaultPrefs[i+1] );
        }
    }
    if (!this->GetValue(kPortProperty[0], tmpValue))
    {
    	for (int j = 1; j< (kNumPorts + 1); j++)
    		this->SetValueByIndex(kPortProperty[0], kPortProperty[j], j-1);
    }
}

int NetInfoPrefsSource::GetValue(const char* inKey, char* ioValue)
{
	return this->GetValueByIndex(inKey, 0, ioValue);
}

int NetInfoPrefsSource::GetValueByIndex(const char* inKey, UInt32 inIndex, char* ioValue)
{
	ni_status status = NI_OK;
	ni_namelist nameList = {};
	void* localDomain = NULL;
	ni_id qtssDir = {};	
	
	ioValue[0] = '\0';

	status = ni_open(NULL, ".", &localDomain);
	if (status != NI_OK)
		return false;
		
	if (status == NI_OK)
		status = ni_pathsearch(localDomain, &qtssDir, gQTSSPropertiesPath);
		
	if (status == NI_OK)
		status = ni_lookupprop(localDomain, &qtssDir, inKey, &nameList);

	if (status == NI_OK)
	{
		if (nameList.ni_namelist_len > inIndex)
			strcpy(ioValue, nameList.ni_namelist_val[inIndex]);
		else
			status = NI_BADID;
		ni_namelist_free(&nameList);
	}
	
	ni_free(localDomain);
	
	return (status == NI_OK);
}

void NetInfoPrefsSource::SetValue(char* inKey, char* inValue)
{
	this->SetValueByIndex(inKey, inValue, NI_INDEX_NULL);
}

void NetInfoPrefsSource::SetValueByIndex(char* inKey, char* inValue, UInt32 inIndex)
{
	void* localDomain = NULL;
	ni_status status = NI_OK;
	ni_namelist nameList = {};

	//ni_namelist_insert(&nameList, (char*)inValue, NI_INDEX_NULL);
	ni_namelist_insert(&nameList, inValue, inIndex);

	status = ni_open(NULL, ".", &localDomain);

	if (status == NI_OK)
	{
		//create the path if it doesn't already exist
		status = ni2_create(localDomain, gQTSSPropertiesPath);

		if (status == NI_OK)
			status = ni2_appendprop(localDomain, gQTSSPropertiesPath, inKey, nameList);
	}
	ni_namelist_free(&nameList);
	ni_free(localDomain);
}
