/*
 * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * Copyright (c) 1999 Apple Computer, Inc.  All Rights Reserved.
 * The contents of this file constitute Original Code as defined in and are 
 * subject to the Apple Public Source License Version 1.1 (the "License").  
 * You may not use this file except in compliance with the License.  Please 
 * obtain a copy of the License at http://www.apple.com/publicsource and 
 * read it before using this file.
 * 
 * This Original Code and all software distributed under the License are 
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, 
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS 
 * FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the License for 
 * the specific language governing rights and limitations under the 
 * License.
 * 
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
/*
	File:		RTSPClient.h

	Contains:	.  
					
	$Log: RTSPClient.h,v $
	Revision 1.2  2000/01/27 02:09:12  rtucker
	Merge changes from v81
	
	Revision 1.4  2000/01/26 21:32:42  serenyi
	Completed the implementation
	
	Revision 1.3  1999/12/21 01:44:02  lecroy
	*** empty log message ***
	
	Revision 1.2  1999/09/30 00:28:52  serenyi
	Continuing development...
	
	
*/

#ifndef __RTSP_CLIENT_H__
#define __RTSP_CLIENT_H__

#include "OSHeaders.h"
#include "StrPtrLen.h"
#include "TCPSocket.h"

class RTSPClient
{
	public:
	
		RTSPClient(UInt32 inSocketType);
		~RTSPClient();
		
		// This must be called before any other function. Sets up very important info.
		void		Set(UInt32 hostAddr, UInt16 hostPort, const StrPtrLen& inURL);
		
		// Send specified RTSP request to server, wait for complete response.
		// These return EAGAIN if transaction is in progress, OS_NoErr if transaction
		// was successful, EINPROGRESS if connection is in progress, or some other
		// error if transaction failed entirely.
		OS_Error	SendDescribe();
		OS_Error	SendSetup(UInt32 inTrackID, UInt16 inClientPort);
		OS_Error	SendPlay();
		OS_Error	SendTeardown();
		
		// ACCESSORS

		UInt32		GetHostAddr()			{ return fHostAddr; }
		UInt32		GetLocalAddr()			{ return fSocket.GetLocalAddr(); }
		StrPtrLen*	GetURL()				{ return &fURL; }
		UInt32		GetStatus() 			{ return fStatus; }
		StrPtrLen*	GetSessionID() 			{ return &fSessionID; }
		UInt16		GetServerPort() 		{ return fServerPort; }
		UInt32		GetContentLength() 		{ return fContentLength; }
		char*		GetContentBody() 		{ return fRecvContentBuffer; }
	
		// If one of the above methods returns EWOULDBLOCK or EINPROGRESS, you
		// can check this to see what events you should wait for on the socket
		UInt32		GetEventMask()			{ return fEventMask; }
		TCPSocket*	GetSocket()				{ return &fSocket; }
	
	private:
	
		// Helper methods
		OS_Error 	DoTransaction();
		OS_Error 	CheckConnection();
		OS_Error 	SendRequest();

		// Call this to receive an RTSP response from the server.
		// Returns EAGAIN until a complete response has been received.
		OS_Error 	ReceiveResponse();
		
		TCPSocket	fSocket;

		// Information we need to send the request
		UInt32		fHostAddr;
		UInt16		fHostPort;
		StrPtrLen	fURL;
		UInt32		fCSeq;
		
		// Response data we get back
		UInt32		fStatus;
		StrPtrLen	fSessionID;
		UInt16		fServerPort;
		UInt32		fContentLength;
		
		enum
		{
			kReqBufSize = 256
		};
		
		// Data buffers
		char		fSendBuffer[kReqBufSize + 1]; 	// for sending requests
		char		fRecvHeaderBuffer[kReqBufSize + 1];// for receiving response headers
		char*		fRecvContentBuffer;				// for receiving response body
		
		// Tracking the state of our sends
		UInt32		fSendBufferLen;
		UInt32		fLengthSent;
		
		// Tracking the state of our receives
		UInt32		fContentRecvLen;
		UInt32		fHeaderRecvLen;
		
		// States
		Bool16		fTransactionStarted;
		Bool16		fReceiveInProgress;
		Bool16		fReceivedResponse;
		
		UInt32		fEventMask;
};

#endif //__RTSP_CLIENT_H__
