/*
 * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * Copyright (c) 1999 Apple Computer, Inc.  All Rights Reserved.
 * The contents of this file constitute Original Code as defined in and are 
 * subject to the Apple Public Source License Version 1.1 (the "License").  
 * You may not use this file except in compliance with the License.  Please 
 * obtain a copy of the License at http://www.apple.com/publicsource and 
 * read it before using this file.
 * 
 * This Original Code and all software distributed under the License are 
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, 
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS 
 * FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the License for 
 * the specific language governing rights and limitations under the 
 * License.
 * 
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
/*
	File:		QTSSCallbacks.h

	Contains:	All the QTSS callback functions
					
	$Log: QTSSCallbacks.h,v $
	Revision 1.2  2000/01/27 02:09:13  rtucker
	Merge changes from v81
	
	Revision 1.7  2000/01/26 21:32:15  serenyi
	Added new QTSS_RequestIdleTime callback, made it so the ClientSessionClosing role can register for async events
	
	Revision 1.6  2000/01/19 19:37:03  serenyi
	Moved to new NEW macro
	
	Revision 1.5  1999/12/21 01:44:03  lecroy
	*** empty log message ***
	
	Revision 1.4  1999/11/08 22:52:33  serenyi
	Updated code to reflect changes in QTSS API
	
	Revision 1.3  1999/10/07 08:52:43  serenyi
	Changed bool to Bool16, changed QTSS_Dictionary to QTSS_Object
	
	Revision 1.2  1999/09/24 19:55:22  murata
	Support for API callQTSS_MilliSecsTo1970Secs.
	
	Revision 1.1.1.1  1999/09/22 23:01:50  lecroy
	importing
	
	Revision 1.1.1.1  1999/09/13 16:16:52  rtucker
	initial checkin from rrv14
	
	

	Created: Tue, Mar 2, 1999 @ 4:23 PM
*/


#ifndef __QTSSCALLBACKS_H__
#define __QTSSCALLBACKS_H__

#include "QTSS.h"

class QTSSCallbacks
{
	public:
	
		// MEMORY ROUTINES
		
		static void* 	QTSS_New(FourCharCode inMemoryIdentifier, UInt32 inSize);
		static void		QTSS_Delete(void* inMemory);
		
		
		// TIME ROUTINES
		static void	QTSS_Milliseconds(SInt64* outMilliseconds);
		static void	QTSS_ConvertToUnixTime(SInt64* inQTSS_MilliSecondsPtr, time_t* outSecondsPtr);

		// STARTUP ROUTINES
		
		static QTSS_Error 	QTSS_AddRole(QTSS_Role inRole);

		// DICTIONARY ROUTINES
		
		static QTSS_Error	QTSS_AddAttribute(QTSS_ObjectType inType, const char* inTag, QTSS_AttrFunctionPtr inFunctionPtr);
		static QTSS_Error	QTSS_IDForAttr(QTSS_ObjectType inType, const char* inTag, QTSS_AttributeID* outID);
		static QTSS_Error 	QTSS_GetValuePtr (QTSS_Object inDictionary, QTSS_AttributeID inID, UInt32 inIndex, void** outBuffer, UInt32* outLen);
		static QTSS_Error 	QTSS_GetValue (QTSS_Object inDictionary, QTSS_AttributeID inID, UInt32 inIndex, void* ioBuffer, UInt32* ioLen);
		static QTSS_Error 	QTSS_SetValue (QTSS_Object inDictionary, QTSS_AttributeID inID, UInt32 inIndex, const void* inBuffer,  UInt32 inLen);

		// STREAM ROUTINES
		
		static QTSS_Error	QTSS_Write(QTSS_StreamRef inStream, void* inBuffer, UInt32 inLen, UInt32* outLenWritten, UInt32 inFlags);
		static QTSS_Error	QTSS_WriteV(QTSS_StreamRef inStream, iovec* inVec, UInt32 inNumVectors, UInt32 inTotalLength, UInt32* outLenWritten);
		static QTSS_Error	QTSS_Flush(QTSS_StreamRef inStream);

		// SERVICE ROUTINES
		
		static QTSS_Error	QTSS_AddService(const char* inServiceName, QTSS_ServiceFunctionPtr inFunctionPtr);
		static QTSS_Error	QTSS_IDForService(const char* inTag, QTSS_ServiceID* outID);
		static QTSS_Error	QTSS_DoService(QTSS_ServiceID inID, QTSS_ServiceFunctionArgsPtr inArgs);

		// RTSP ROUTINES
		
		static QTSS_Error 	QTSS_SendRTSPHeaders(QTSS_RTSPRequestObject inRef);
		static QTSS_Error 	QTSS_AppendRTSPHeader(QTSS_RTSPRequestObject inRef, QTSS_RTSPHeader inHeader, char* inValue, UInt32 inValueLen);
		static QTSS_Error 	QTSS_SendStandardRTSPResponse(QTSS_RTSPRequestObject inRTSPRequest, QTSS_Object inRTPInfo, UInt32 inFlags);

		// RTP ROUTINES
		
		static QTSS_Error	QTSS_AddRTPStream(QTSS_ClientSessionObject inClientSession, QTSS_RTSPRequestObject inRTSPRequest, QTSS_RTPStreamObject* outStream, QTSS_AddStreamFlags inFlags);
		static QTSS_Error	QTSS_Play(QTSS_ClientSessionObject inClientSession, QTSS_RTSPRequestObject inRTSPRequest, QTSS_PlayFlags inPlayFlags);
		static QTSS_Error	QTSS_Pause(QTSS_ClientSessionObject inClientSession);
		static QTSS_Error	QTSS_Teardown(QTSS_ClientSessionObject inClientSession);
		
		// ASYNC I/O ROUTINES
		static QTSS_Error	QTSS_CreateEventContext(int inFileDesc, QTSS_EventContextRef* outContext, void* inPrivate); 
		static QTSS_Error	QTSS_DestroyEventContext(QTSS_EventContextRef inContext);

		static QTSS_Error	QTSS_RequestEvent(QTSS_EventContextRef inContext, int inEventMask);
		static QTSS_Error	QTSS_SetIdleTimer(SInt64 inMsecToWait);

	private:
	
};

#endif //__QTSSCALLBACKS_H__
