/*
 * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * Copyright (c) 1999 Apple Computer, Inc.  All Rights Reserved.
 * The contents of this file constitute Original Code as defined in and are 
 * subject to the Apple Public Source License Version 1.1 (the "License").  
 * You may not use this file except in compliance with the License.  Please 
 * obtain a copy of the License at http://www.apple.com/publicsource and 
 * read it before using this file.
 * 
 * This Original Code and all software distributed under the License are 
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, 
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS 
 * FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the License for 
 * the specific language governing rights and limitations under the 
 * License.
 * 
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
/*
	File:		QTSSMessages.cpp

	Contains:	Implementation of object defined in .h
	
	$Log: QTSSMessages.cpp,v $
	Revision 1.2  2000/01/27 02:09:13  rtucker
	Merge changes from v81
	
	Revision 1.8  2000/01/26 18:33:25  serenyi
	#2426075 Integrated changes from Kostya Vasilyev
	
	Revision 1.7  2000/01/21 23:52:55  rtucker
	
	logging changes
	
	Revision 1.6  2000/01/19 19:37:04  serenyi
	Moved to new NEW macro
	
	Revision 1.5  1999/12/21 01:44:04  lecroy
	*** empty log message ***
	
	Revision 1.4  1999/11/04 22:33:17  serenyi
	Added some new error messages
	
	Revision 1.3  1999/10/28 01:45:44  serenyi
	Added a new error message
	
	Revision 1.2  1999/10/06 07:17:38  serenyi
	Added new built-in server message
	
	Revision 1.1.1.1  1999/09/22 23:01:51  lecroy
	importing
	
	Revision 1.1.1.1  1999/09/13 16:16:53  rtucker
	initial checkin from rrv14
	
	
	Revision 1.2  1999/02/19 23:08:28  ds
	Created
	
	
*/

#include "QTSSMessages.h"
#include "QTSSModuleInterface.h"
#include "OSMemory.h"

// see QTSS.h (QTSS_TextMessagesObject) for list of enums to map these strings

char*		QTSSMessages::sMessagesKeyStrings[] =
{
/* 0 */	"qtssMsgNoMessage",
/* 1 */	"qtssMsgNoURLInRequest",
/* 2 */	"qtssMsgBadRTSPMethod",
/* 3 */	"qtssMsgNoRTSPVersion",
/* 4 */	"qtssMsgNoRTSPInURL",
/* 5 */	"qtssMsgURLTooLong",
/* 6 */	"qtssMsgURLInBadFormat",
/* 7 */	"qtssMsgNoColonAfterHeader",
/* 8 */	"qtssMsgNoEOLAfterHeader",
/* 9 */	"qtssMsgRequestTooLong",
/* 10*/	"qtssMsgNoModuleFolder",
/* 11*/	"qtssMsgCouldntListen",
/* 12*/	"qtssMsgInitFailed",
/* 13*/	"qtssMsgNotConfiguredForIP",
/* 14*/	"qtssMsgDefaultRTSPAddrUnavail",
/* 15*/	"qtssMsgBadModule",
/* 16*/	"qtssMsgRegFailed",
/* 17*/	"qtssMsgRefusingConnections",
/* 18*/	"qtssMsgTooManyClients",
/* 19*/	"qtssMsgTooMuchThruput",
/* 20*/	"qtssMsgNoSessionID",
/* 21*/	"qtssMsgFileNameTooLong",
/* 22*/	"qtssMsgNoClientPortInTransport",
/* 23*/	"qtssMsgRTPPortMustBeEven",
/* 24*/	"qtssMsgRTCPPortMustBeOneBigger",
/* 25*/	"qtssMsgOutOfPorts",
/* 26*/	"qtssMsgNoModuleForRequest",
/* 27*/	"qtssMsgAltDestNotAllowed",
/* 28*/	"qtssMsgCantSetupMulticast",
/* 29*/	"qtssListenPortInUse",
/* 30*/	"qtssListenPortAccessDenied",
/* 31*/	"qtssListenPortError",
/* 32*/	"qtssMsgBadBase64",
/* 33 */ "qtssMsgSomePortsFailed",
/* 34 */ "qtssMsgNoPortsSucceeded",
/* 35*/ "qtssMsgCannotCreatePidFile",
/* 36*/ "qtssMsgCannotSetRunUser",
/* 37*/ "qtssMsgCannotSetRunGroup",


 // module specific messages follow (these are dynamically numbered)
 
	"QTSSvrControlModuleCantRegisterMachPort",
	"QTSSvrControlModuleCantAllocateMachPort",
	"QTSSvrControlModuleServerControlFatalErr",
	"QTSSReflectorModuleCantBindReflectorSocket",
	"QTSSReflectorModuleCantJoinMulticastGroup",
	"QTSSFileModuleSeekToNonExistentTime",
	"QTSSFileModuleNoSDPFileFound",
	"QTSSFileModuleBadQTFile",
	"QTSSFileModuleFileIsNotHinted",
	"QTSSFileModuleExpectedDigitFilename",
	"QTSSFileModuleTrackDoesntExist",
	"QTSSReflectorModuleExpectedDigitFilename",
	"QTSSSpamDefenseModuleTooManyConnections",
	"QTSSReflectorModuleBadTrackID",
	"QTSSAccessModuleBadAccessFileName",
	"QTSSReflectorModuleNoRelaySources",
	"QTSSReflectorModuleNoRelayDests",
	"QTSSReflectorModuleNoRelayStreams",
	"QTSSReflectorModuleNoRelayConfig"
};

// see QTSS.h (QTSS_TextMessagesObject) for list of enums to map these strings

char*		QTSSMessages::sMessages[] =
{
/* 0 */	"No Message",
/* 1 */	"There was no URL contained in the following request: %s",
/* 2 */	"The following RTSP method: %s, was not understood by the server",
/* 3 */	"There is no RTSP version in the following request: %s",
/* 4 */	"Server expected 'rtsp://' and instead received: %s",
/* 5 */	"The following URL is too long to be processed by the server: %s",
/* 6 */	"The following URL is not in proper URL format: %s",
/* 7 */	"There was no colon after a header in the following request: %s",
/* 8 */	"There was no EOL after a header in the following request: %s",
/* 9 */	"That request is too long to be processed by the server.",
/* 10*/	"No module folder exists.",
/* 11*/	"Streaming Server couldn't listen on a specified RTSP port. Quitting.",
/* 12*/	"The module %s failed to Initialize.",
/* 13*/	"This machine is currently not configured for IP.",
/* 14*/	"The specified RTSP listening IP address doesn't exist.",
/* 15*/	"The module %s is not a compatible QTSS API module.",
/* 16*/	"The module %s failed to Register.",
/* 17*/	"Streaming Server is currently refusing new connections",
/* 18*/	"Too many clients connected",
/* 19*/	"Too much bandwidth being served",
/* 20*/	"No active RTP session for that RTSP session ID",
/* 21*/	"Specified file name is too long to be handled by the server.",
/* 22*/	"No client port pair found in transport header",
/* 23*/	"Reported client RTP port is not an even number",
/* 24*/	"Reported client RTCP port is not one greater than RTP port",
/* 25*/	"Streaming Server couldn't find any available UDP ports",
/* 26*/	"There is no QTSS API module installed to process this request.",
/* 27*/	"Not allowed to specify an alternate destination address",
/* 28*/	"Can't setup multicast.",
/* 29*/	"Another process is already using the following RTSP port: %s",
/* 30*/	"You must be root to use the following RTSP port: %s",
/* 31*/	"An error occurred when attempting to listen on the following RTSP port: %s",
/* 32*/	"The base64 you just sent to the server is corrupt!",
/* 33*/ "Streaming Server failed to listen on all requested RTSP port(s).",
/* 34*/ "Streaming Server is not listening for RTSP on any ports.",
/* 35*/ "Error creating pid file %s: %s",
/* 36*/ "Error switching to user %s: %s",
/* 37*/ "Error switching to group %s: %s",


// module specific messages follow (these are dynamically numbered)

/* 38*/	"Fatal error: Can't register Mach Ports.",
/* 39*/	"Fatal error: Can't register Mach Ports.",
/* 40*/	"A fatal error occcured while starting up server control module",
/* 41*/	"Can't bind reflector sockets",
/* 42*/	"Reflector sockets couldn't join multicast",
/* 43*/	"Couldn't seek to specified time.",
/* 44*/	"No SDP file found for the following URL: %s",
/* 45*/	"The requested file is not a movie file.",
/* 46*/	"Requested movie hasn't been hinted.",
/* 47*/	"Expected a digit at the end of the following URL: %s",
/* 48*/	"Specified trackID doesn't exist in the movie",
/* 49*/	"Expected a digit at the end of the following URL: %s",
/* 50*/	"Too many connections from your IP address!",
/* 51*/	"TrackID doesn't match any trackID for this ReflectorSession",
/* 52*/	"Invalid config value for qtaccessfilename: name contains %s. Now using default name:%s",
/* 53*/	"The relay configuration file at: %s has no relay_source lines",
/* 54*/	"Could not find any relay_destination lines for one of the relay_source lines of the relay configuration file",
/* 55*/	"Could not find any input stream information for one of the relay_source lines of the relay configuration file",
/* 56*/	"Found an empty relay configuration file at %s"
};

// need to maintain numbers to update kNumMessages in QTSSMessages.h.

void QTSSMessages::Initialize()
{
	QTSSDictionaryMap* theMap = QTSSModuleInterface::GetDictionaryMap(QTSSModuleInterface::kTextMessagesDictIndex);
	Assert(theMap != NULL);
	
	for (UInt32 x = 0; x < qtssMsgNumParams; x++)
		theMap->SetAttribute(x, sMessagesKeyStrings[x], NULL, true, false);
}

QTSSMessages::QTSSMessages(PrefsSource* inMessages)
: QTSSDictionary(QTSSModuleInterface::GetDictionaryMap(QTSSModuleInterface::kTextMessagesDictIndex))
{
	static const UInt32 kMaxMessageSize = 2048;
	char theMessage[kMaxMessageSize];
	
	// Use the names of the attributes in the attribute map as the key values for
	// finding preferences in the config file.
	
	for (UInt32 x = 0; x < this->GetDictionaryMap()->GetNumAttrs(); x++)
	{
		theMessage[0] = '\0';
		(void)inMessages->GetValue(this->GetDictionaryMap()->GetAttrName(x), &theMessage[0]);
		
		if (theMessage[0] == '\0')
		{
			// If a message doesn't exist in the file, check to see if this attribute
			// name matches one of the compiled-in strings. If so, use that instead
			for (UInt32 y = 0; y < kNumMessages; y++)
			{
				if (::strcmp(this->GetDictionaryMap()->GetAttrName(x), sMessagesKeyStrings[y]) == 0)
				{
					::strcpy(theMessage, sMessages[y]);
					break;
				}
			}
			// If we didn't find a match, just copy in this last-resort message
			if (theMessage[0] == '\0')
				::strcpy(theMessage, "No Message");
		}
		
		// Add this preference into the dictionary.
		
		UInt32 theID = this->GetDictionaryMap()->GetAttrID(x);

		// If there is a message, allocate some new memory for 
		// the new attribute, and copy the data into the newly allocated buffer
		if (theMessage[0] != '\0')
		{
			char* attrBuffer = NEW char[::strlen(theMessage) + 2];
			::strcpy(attrBuffer, theMessage);
			this->SetVal(theID, attrBuffer, ::strlen(attrBuffer));
		}
	}
}
