/*
 * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * Copyright (c) 1999 Apple Computer, Inc.  All Rights Reserved.
 * The contents of this file constitute Original Code as defined in and are 
 * subject to the Apple Public Source License Version 1.1 (the "License").  
 * You may not use this file except in compliance with the License.  Please 
 * obtain a copy of the License at http://www.apple.com/publicsource and 
 * read it before using this file.
 * 
 * This Original Code and all software distributed under the License are 
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, 
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS 
 * FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the License for 
 * the specific language governing rights and limitations under the 
 * License.
 * 
 * 
 * @APPLE_LICENSE_HEADER_END@

	Contains:	Object store for RTSP server preferences.
	
	$Log: QTSSPrefs.h,v $
	Revision 1.2  2000/01/27 02:09:13  rtucker
	Merge changes from v81
	
	Revision 1.11  2000/01/26 18:33:25  serenyi
	#2426075 Integrated changes from Kostya Vasilyev
	
	Revision 1.10  1999/12/21 01:44:05  lecroy
	*** empty log message ***
	
	Revision 1.9  1999/11/11 00:35:43  rtucker
	 removed authentication preference
	
	Revision 1.8  1999/10/19 17:33:21  serenyi
	Added new prefs for the purposes of scaling TCP send buffers, added the ability to set these send buffers
	
	Revision 1.7  1999/10/16 01:52:51  murata
	add changes to parse request for authentication info.
	
	Revision 1.6  1999/10/16 00:57:59  rtucker
	changes fo default vals
	
	Revision 1.5  1999/10/16 00:28:30  murata
	Use new GetPrefs calls
	
	Revision 1.4  1999/10/14 23:18:43  rtucker
	changes to report server ip back to client over HTTP
	
	Revision 1.3  1999/10/08 01:12:50  rtucker
	added support for TCP thining and thinnig prefs
	
	
	
	
	VS:
	
	Revision 1.2  1999/10/07 08:52:44  serenyi
	Changed bool to Bool16, changed QTSS_Dictionary to QTSS_Object
	
	Revision 1.1.1.1  1999/09/22 23:01:51  lecroy
	importing
	
	Revision 1.1.1.1  1999/09/13 16:16:53  rtucker
	initial checkin from rrv14
	
	
	Revision 1.2  1999/02/19 23:08:30  ds
	Created
	
	
*/

#ifndef __QTSSPREFS_H__
#define __QTSSPREFS_H__

#include "QTSS.h"
#include "QTSSDictionary.h"
#include "QTSSModuleUtils.h"

#include "StrPtrLen.h"
#include "PrefsSource.h"

class QTSSPrefs : public QTSSDictionary
{
	public:

		// INITIALIZE
		//
		// This function sets up the dictionary map. Must be called before instantiating
		// the first RTSPPrefs object.
	
		static void Initialize();

		//Arguments:	inPrefsSource: an object to fetch preferences from
		//				inLogTimestamps:	Specify "log timestamps" setting.
		//				inAppendRandomOffsets: Specify "append random offsets" setting
		QTSSPrefs(PrefsSource* inPrefsSource);
		virtual ~QTSSPrefs() {}
		
		//This is callable at any time, and is thread safe wrt to the accessors
		void RereadPreferences();

		//Prefs source: if you want raw access to the source of the preferences
		PrefsSource* 	GetPrefsSource()	{ return fPrefsSource; }
		OSMutex*		GetMutex()			{ return &fPrefsMutex; }
		
		//Individual accessor methods for preferences.
		
		//this may return INADDR_ANY
		UInt32	GetRTSPIPAddress() 						{ return fIPAddress; }
		
		//Amount of idle time after which respective protocol sessions are timed out
		//(stored in seconds)
		
		//This is the value we advertise to clients (lower than the real one)
		UInt32	GetRTSPTimeoutInSecs()	{ return fRTSPTimeoutInSecs; }
		UInt32	GetRTPTimeoutInSecs()	{ return fRTPTimeoutInSecs; }
		StrPtrLen*	GetRTSPTimeoutAsString() { return &fRTSPTimeoutString; }
		
		//This is the real timeout
		UInt32	GetRealRTSPTimeoutInSecs(){ return fRealRTSPTimeoutInSecs; }
		
		//-1 means unlimited
		SInt32	GetMaxConnections()			{ return fMaximumConnections; }
		SInt32	GetMaxKBitsBandwidth()		{ return fMaxBandwidthInKBits; }
		
		// for tcp thinning
		SInt32	GetTCPMinThinDelayToleranceInMSec()		{ return fTCPMinThinDelayToleranceInMSec; }
		SInt32	GetTCPMaxThinDelayToleranceInMSec()		{ return fTCPMaxThinDelayToleranceInMSec; }
		SInt32	GetTCPVideoDelayToleranceInMSec()		{ return fTCPVideoDelayToleranceInMSec;   }
		SInt32	GetTCPAudioDelayToleranceInMSec()		{ return fTCPAudioDelayToleranceInMSec;   }
		
		// for tcp buffer size scaling
		UInt32	GetMinTCPBufferSizeInBytes()			{ return fMinTCPBufferSizeInBytes; }
		UInt32	GetMaxTCPBufferSizeInBytes()			{ return fMaxTCPBufferSizeInBytes; }
		Float32	GetTCPSecondsToBuffer()					{ return fTCPSecondsToBuffer; }
		
		//for joining HTTP sessions from behind a round-robin DNS
		Bool16	GetDoReportHTTPConnectionAddress() 		{ return fDoReportHTTPConnectionAddress;  }
		
		//for debugging, mainly
		Bool16		ShouldServerBreakOnAssert()			{ return fBreakOnAssert; }
		Bool16		IsAutoRestartEnabled()				{ return fAutoRestart; }

		UInt32		GetTotalBytesUpdateTimeInSecs()		{ return fTBUpdateTimeInSecs; }
		UInt32		GetAvgBandwidthUpdateTimeInSecs()	{ return fABUpdateTimeInSecs; }
		UInt32		GetSafePlayDurationInSecs()			{ return fSafePlayDurationInSecs; }
		
		// For the compiled-in error logging module
		
		Bool16	IsErrorLogEnabled() 	{ return fErrorLogEnabled; }
		Bool16	IsScreenLoggingEnabled(){ return fScreenLoggingEnabled; }

		UInt32	GetMaxErrorLogBytes() 			{ return fErrorLogBytes; }
		UInt32	GetErrorRollIntervalInDays() 	{ return fErrorRollIntervalInDays; }
		UInt32	GetErrorLogVerbosity() 			{ return fErrorLogVerbosity; }

		// Movie folder pref. If the path fits inside the buffer provided,
		// the path is copied into that buffer. Otherwise, a new buffer is allocated
		// and returned.
		char*	GetMovieFolder(char* inBuffer, UInt32* ioLen);
		
		// String preferences. Note that the pointers returned here is allocated
		// memory that you must delete!
		
		char*	GetErrorLogDir()
			{ return QTSSModuleUtils::GetStringPref(qtssPrefsErrorLogDir, sDefaultErrorLogDir); }
		char*	GetErrorLogName()
			{ return QTSSModuleUtils::GetStringPref(qtssPrefsErrorLogName, sDefaultErrorLogName); }

		char*	GetModuleDirectory()
			{ return QTSSModuleUtils::GetStringPref(qtssPrefsModuleFolder, sDefaultModuleFolder); }
			
		char*	GetAuthorizationRealm()
			{ return QTSSModuleUtils::GetStringPref(qtssPrefsDefaultAuthorizationRealm, sDefaultAuthorizationRealm); }

		char*	GetPidFileName()
			{ return QTSSModuleUtils::GetStringPref(qtssPrefsPidFileName, sDefaultPidFileName); }
		char*	GetRunUserName()
			{ return QTSSModuleUtils::GetStringPref(qtssPrefsRunUserName, sDefaultRunUserName); }
		char*	GetRunGroupName()
			{ return QTSSModuleUtils::GetStringPref(qtssPrefsRunGroupName, sDefaultRunGroupName); }


	private:

		PrefsSource* fPrefsSource;

		UInt32 		fRTSPTimeoutInSecs;
		char		fRTSPTimeoutBuf[20];
		StrPtrLen	fRTSPTimeoutString;
		UInt32 		fRealRTSPTimeoutInSecs;
		UInt32 		fRTPTimeoutInSecs;
		
		SInt32 	fMaximumConnections;
		SInt32	fMaxBandwidthInKBits;
		
		UInt32	fIPAddress;
		Bool16	fBreakOnAssert;
		Bool16	fAutoRestart;
		UInt32	fTBUpdateTimeInSecs;
		UInt32	fABUpdateTimeInSecs;
		UInt32	fSafePlayDurationInSecs;
		
		UInt32	fErrorRollIntervalInDays;
		UInt32	fErrorLogBytes;
		UInt32	fErrorLogVerbosity;
		Bool16	fScreenLoggingEnabled;
		Bool16	fErrorLogEnabled;

		// tcp (http) thinning parameters
		SInt32	fTCPMinThinDelayToleranceInMSec;
		SInt32	fTCPMaxThinDelayToleranceInMSec;
		SInt32	fTCPVideoDelayToleranceInMSec;
		SInt32	fTCPAudioDelayToleranceInMSec;

		UInt32	fMinTCPBufferSizeInBytes;
		UInt32	fMaxTCPBufferSizeInBytes;
		Float32	fTCPSecondsToBuffer;

		Bool16	fDoReportHTTPConnectionAddress;
		
		OSMutex	fPrefsMutex;
						
		//Helper functions for RereadPreferences()
		void ProcessBooleanPref(char* inBuffer, Bool16* inPref, Bool16 inDefault);
		void ProcessUIntPref(char* inBuffer, UInt32* inPref, UInt32 inDefault);
		void ProcessSIntPref(char* inBuffer, SInt32* inPref, SInt32 inDefault);
		void ProcessFloat32Pref(char* inBuffer, Float32* inPref, Float32 inDefault);
		void SetupAttributes();

		void	ProcessStringPref(UInt32 inID, char* inNewValue, char* inDefaultValue);
		
		//There needs to be separate storage of defaults from standard prefs, because
		//if user removes some pref from the file and asks server to reread, we need
		//to go back to a default.
		
		static UInt32 	sDefaultRTSPTimeout;
		static UInt32	sDefaultRealRTSPTimeoutInSecs;
		static UInt32 	sDefaultRTPTimeout;
		static SInt32 	sDefaultMaximumConnections;
		static SInt32	sDefaultMaxBandwidth;
		static char*  	sDefaultMoviesFolder;
		static UInt32	sDefaultIPAddress;
		static char*	sDefaultIPAddressStr;
		static Bool16	sDefaultBreakOnAssert;
		static Bool16	sDefaultAutoRestart;
		static UInt32	sDefaultTBUpdateTime;
		static UInt32	sDefaultABUpdateTime;
		static UInt32	sDefaultSafePlayDuration;
		static char*	sDefaultModuleFolder;

		static char*	sDefaultErrorLogName;
		static char*	sDefaultErrorLogDir;
		static UInt32	sErrorRollIntervalInDays;
		static UInt32	sErrorLogBytes;
		static UInt32	sErrorLogVerbosity;

		static SInt32	sDefaultTCPMinThinDelayToleranceInMSec;
		static SInt32	sDefaultTCPMaxThinDelayToleranceInMSec;
		static SInt32	sDefaultTCPVideoDelayToleranceInMSec;
		static SInt32	sDefaultTCPAudioDelayToleranceInMSec;

		static UInt32	sDefaultMinTCPBufferSizeInBytes;
		static UInt32	sDefaultMaxTCPBufferSizeInBytes;
		static Float32	sDefaultTCPSecondsToBuffer;

		static Bool16	sScreenLoggingEnabled;
		static Bool16	sErrorLogEnabled;
		
		static Bool16	sDefaultDoReportHTTPConnectionAddress;
		
		static char* 	sEnabledString;
		static char* 	sPrefsKeyStrings[];

		static char*	sDefaultAuthorizationRealm;
		static Bool16	sDefaultAuthorizationEnabled;

		static char*	sDefaultPidFileName;
		static char*	sDefaultRunUserName;
		static char*	sDefaultRunGroupName;
};
#endif //__QTSSPREFS_H__
