/*
 * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * Copyright (c) 1999 Apple Computer, Inc.  All Rights Reserved.
 * The contents of this file constitute Original Code as defined in and are 
 * subject to the Apple Public Source License Version 1.1 (the "License").  
 * You may not use this file except in compliance with the License.  Please 
 * obtain a copy of the License at http://www.apple.com/publicsource and 
 * read it before using this file.
 * 
 * This Original Code and all software distributed under the License are 
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, 
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS 
 * FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the License for 
 * the specific language governing rights and limitations under the 
 * License.
 * 
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
/*
	File:		QTSSStream.h

	Contains:	Abstract base class containing the prototypes for generalized
				stream functions.
				
				Any server object that wants to act as a QTSS_StreamRef should
				derive off of this and implement one or more of the stream APIs.
					
	$Log: QTSSStream.h,v $
	Revision 1.1.1.1  2000/01/17 23:36:03  wsanchez
	Import of StreamingServer-2.0 (Apple)
	
	Revision 1.2  1999/12/21 01:44:05  lecroy
	*** empty log message ***
	
	Revision 1.1.1.1  1999/09/22 23:01:52  lecroy
	importing
	
	Revision 1.1.1.1  1999/09/13 16:16:53  rtucker
	initial checkin from rrv14
	
	

	Created: Tue, Mar 2, 1999 @ 4:23 PM
*/


#ifndef __QTSS_STREAM_H__
#define __QTSS_STREAM_H__

#include "QTSS.h"

class QTSSStream
{
	public:

		virtual QTSS_Error	Read(void* /*ioBuffer*/, UInt32 /*inLen*/, UInt32* /*outLen*/)
															{ return QTSS_Unimplemented; }
															
		virtual QTSS_Error	Write(void* /*inBuffer*/, UInt32 /*inLen*/, UInt32* /*outLenWritten*/, UInt32 /*inFlags*/)
															{ return QTSS_Unimplemented; }
		
		virtual QTSS_Error	WriteV(iovec* /*inVec*/, UInt32 /*inNumVectors*/, UInt32 /*inTotalLength*/, UInt32* /*outLenWritten*/)
															{ return QTSS_Unimplemented; }
															
		virtual QTSS_Error	Flush() { return QTSS_Unimplemented; }
		
};

#endif //__QTSS_STREAM_H__
