/*
 * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * Copyright (c) 1999 Apple Computer, Inc.  All Rights Reserved.
 * The contents of this file constitute Original Code as defined in and are 
 * subject to the Apple Public Source License Version 1.1 (the "License").  
 * You may not use this file except in compliance with the License.  Please 
 * obtain a copy of the License at http://www.apple.com/publicsource and 
 * read it before using this file.
 * 
 * This Original Code and all software distributed under the License are 
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, 
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS 
 * FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the License for 
 * the specific language governing rights and limitations under the 
 * License.
 * 
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
/*
	File:		QTSServerInterface.cpp

	Contains:	Implementation of object defined in QTSServerInterface.h.
	
	$Log: QTSServerInterface.cpp,v $
	Revision 1.2  2000/01/27 02:09:14  rtucker
	Merge changes from v81
	
	Revision 1.27  2000/01/19 19:37:05  serenyi
	Moved to new NEW macro
	
	Revision 1.26  1999/12/21 01:44:06  lecroy
	*** empty log message ***
	
	Revision 1.25  1999/12/17 03:22:28  murata
	Remove the change to use DarwingStreamingServer for server name. Not enough time to fully test.
	
	Revision 1.24  1999/12/17 02:10:12  murata
	User "DarwinStreamingServer" for server name dictionary item on non MacOSX systems.
	
	Revision 1.23  1999/12/02 19:26:26  serenyi
	Got rid of "RoadRunner", added a platform string to the Server: header
	
	Revision 1.22  1999/10/30 07:13:33  serenyi
	Build environment slightly different on linux, it now works
	
	Revision 1.21  1999/10/27 23:51:36  rtucker
	uses revision.h for bulid and version info
	
	Revision 1.20  1999/10/27 03:41:19  lecroy
	updated vnum
	
	Revision 1.19  1999/10/26 00:13:12  serenyi
	Upped version
	
	Revision 1.18  1999/10/22 23:57:46  rtucker
	
	uppped version to 46
	
	Revision 1.17  1999/10/22 22:27:57  rtucker
	upped rev to v45
	
	Revision 1.16  1999/10/19 22:20:45  rtucker
	RoadRunner 42
	
	Revision 1.15  1999/10/15 21:54:17  lecroy
	fixed up serverapi attribute
	
	Revision 1.14  1999/10/14 17:59:46  lecroy
	Added qtssAPIVersion attribute
	
	Revision 1.13  1999/10/12 22:54:24  serenyi
	Updated version
	
	Revision 1.12  1999/10/07 08:52:31  serenyi
	Changed bool to Bool16, changed QTSS_Dictionary to QTSS_Object
	
	Revision 1.11  1999/10/07 01:09:18  lecroy
	version# change
	
	Revision 1.10  1999/10/05 00:34:11  lecroy
	updated ReadMe's and vers#
	
	Revision 1.9  1999/10/02 00:56:28  serenyi
	updated version
	
	Revision 1.8  1999/10/01 19:44:52  rtucker
	upped version for alpha seed buld
	wq
	
	Revision 1.7  1999/09/30 00:47:22  serenyi
	Upped version
	
	Revision 1.6  1999/09/30 00:30:39  serenyi
	Added support for HTTP session attribute
	
	Revision 1.5  1999/09/27 22:49:45  rtucker
	change install to directory, change rev
	
	Revision 1.4  1999/09/24 21:21:47  rtucker
	update for client debugging of TCP flow contro
	
	Revision 1.3  1999/09/24 20:34:00  murata
	new version 27
	
	Revision 1.2  1999/09/23 00:18:41  lecroy
	changed to v27
	
	Revision 1.1.1.1  1999/09/22 23:01:50  lecroy
	importing
	
	Revision 1.10  1999/09/17 23:13:15  serenyi
	updated version
	
	Revision 1.9  1999/09/17 21:50:56  lecroy
	updated version# to v23
	
	Revision 1.8  1999/09/17 03:03:32  serenyi
	udpated version
	
	Revision 1.7  1999/09/16 23:50:19  lecroy
	updated vers#
	
	Revision 1.6  1999/09/16 01:48:26  lecroy
	updated to v# to v20
	
	Revision 1.5  1999/09/15 21:41:30  rtucker
	for buld 18
	
	Revision 1.4  1999/09/14 22:19:05  john
	change to 17
	
	Revision 1.3  1999/09/14 02:29:24  murata
	change version to 16 for Authentication fix for broadcasts (.sdp files)
	
	Revision 1.2  1999/09/14 00:46:29  murata
	fix bugs: RTSPSession.cpp now not returning error if not HTTP stream
	
	Revision 1.2  1999/02/19 23:08:36  ds
	Created
	

*/

//INCLUDES:
#include "QTSServerInterface.h"
#include "QTSSModuleInterface.h"

#include "RTPSessionInterface.h"
#include "OSRef.h"
#include "UDPSocketPool.h"
#include "RTSPProtocol.h"
#ifndef __MacOSX__
#include "revision.h"
#endif

// STATIC DATA

UInt32					QTSServerInterface::sServerAPIVersion = QTSS_API_VERSION;
QTSServerInterface*		QTSServerInterface::sServer = NULL;
StrPtrLen 				QTSServerInterface::sServerNameStr("QTSS");

// kVersionString from revision.h, include with -i at project level
StrPtrLen 				QTSServerInterface::sServerVersionStr(kVersionString);
StrPtrLen 				QTSServerInterface::sServerPlatformStr(kPlatformNameString);
StrPtrLen 				QTSServerInterface::sServerBuildDateStr(__DATE__ " , "__TIME__);
char					QTSServerInterface::sServerHeader[kMaxServerHeaderLen];
StrPtrLen				QTSServerInterface::sServerHeaderPtr(sServerHeader, kMaxServerHeaderLen);


QTSSDictionaryMap::AttrInfo	QTSServerInterface::sAttributes[] = 
{   /*fields:   fAttrName, fFuncPtr, fIsPreemptiveSafe, fIsWriteable */
	/* 0  */ { "qtssServerAPIVersion", 			NULL, 	true,	false },
	/* 1  */ { "qtssSvrDefaultDNSName", 		NULL, 	true,	false },
	/* 2  */ { "qtssSvrDefaultIPAddr", 			NULL, 	true,	false },
	/* 3  */ { "qtssSvrServerName", 			NULL, 	true,	false },
	/* 4  */ { "qtssRTSPSvrServerVersion", 		NULL, 	true,	false },
	/* 5  */ { "qtssRTSPSvrServerBuildDate", 	NULL, 	true,	false },
	/* 6  */ { "qtssSvrRTSPPorts", 				NULL, 	true,	false },
	/* 7  */ { "qtssSvrRTSPServerHeader", 		NULL, 	true,	false },
	/* 8  */ { "qtssRTSPSvrState", 				NULL, 	false,	true  },
	/* 9  */ { "qtssSvrIsOutOfDescriptors", 	IsOutOfDescriptors, 	false,	false },
	/* 10 */ { "qtssRTSPCurrentSessionCount", 	NULL, 	false,	false },
	/* 11 */ { "qtssRTSPHTTPCurrentSessionCount",NULL, 	false,	false },
	/* 12 */ { "qtssRTPSvrNumUDPSockets", 		GetTotalUDPSockets, 	false,	false },
	/* 13 */ { "qtssRTPSvrCurConn", 			NULL, 	false,	false },
	/* 14 */ { "qtssRTPSvrTotalConn", 			NULL, 	false,	false },
	/* 15 */ { "qtssRTPSvrCurBandwidth", 		NULL, 	false,	false },
	/* 16 */ { "qtssRTPSvrTotalBytes", 			NULL, 	false,	false },
	/* 17 */ { "qtssRTPSvrAvgBandwidth", 		NULL, 	false,	false },
	/* 18 */ { "qtssRTPSvrCurPackets", 			NULL, 	false,	false },
	/* 19 */ { "qtssRTPSvrTotalPackets", 		NULL, 	false,	false }
};

void	QTSServerInterface::Initialize()
{
	for (UInt32 x = 0; x < qtssSvrNumParams; x++)
		QTSSModuleInterface::GetDictionaryMap(QTSSModuleInterface::kServerDictIndex)->
			SetAttribute(x, sAttributes[x].fAttrName, sAttributes[x].fFuncPtr,
				sAttributes[x].fIsPreemptiveSafe, sAttributes[x].fIsWriteable);

	//Write out a premade server header
	StringFormatter serverFormatter(sServerHeaderPtr.Ptr, kMaxServerHeaderLen);
	serverFormatter.Put(RTSPProtocol::GetHeaderString(qtssServerHeader));
	serverFormatter.Put(": ");
	serverFormatter.Put(sServerNameStr);
	serverFormatter.PutChar('/');
	serverFormatter.Put(sServerVersionStr);
	serverFormatter.PutChar('-');
	serverFormatter.Put(sServerPlatformStr);
	sServerHeaderPtr.Len = serverFormatter.GetCurrentOffset();
	Assert(sServerHeaderPtr.Len < kMaxServerHeaderLen);
}

QTSServerInterface::QTSServerInterface()
 : 	QTSSDictionary(QTSSModuleInterface::GetDictionaryMap(QTSSModuleInterface::kServerDictIndex), &fMutex),
	fSocketPool(NULL),
	fRTPMap(NULL),
	fSrvrPrefs(NULL),
	fSrvrMessages(NULL),
 	fServerState(qtssStartingUpState),
 	fDefaultIPAddr(0),
 	fListeners(NULL),
 	fNumListeners(0),
 	fMutex(),
 	fNumRTSPSessions(0),
 	fNumRTSPHTTPSessions(0),
	fNumRTPSessions(0),
 	fTotalRTPSessions(0),
 	fTotalRTPBytes(0),
 	fTotalRTPPackets(0),
 	fPeriodicRTPBytes(0),
 	fPeriodicRTPPackets(0),
 	fCurrentRTPBandwidthInBits(0),
 	fAvgRTPBandwidthInBits(0),
 	fRTPPacketsPerSecond(0)
{
	this->SetVal(qtssSvrState, 				&fServerState, 				sizeof(fServerState));
	this->SetVal(qtssServerAPIVersion, 		&sServerAPIVersion, 		sizeof(sServerAPIVersion));
	this->SetVal(qtssSvrDefaultIPAddr, 		&fDefaultIPAddr, 			sizeof(fDefaultIPAddr));
	this->SetVal(qtssSvrServerName, 		sServerNameStr.Ptr, 		sServerNameStr.Len);
	this->SetVal(qtssSvrServerVersion, 		sServerVersionStr.Ptr, 		sServerVersionStr.Len);
	this->SetVal(qtssSvrServerBuildDate, 	sServerBuildDateStr.Ptr, 	sServerBuildDateStr.Len);
	this->SetVal(qtssSvrRTSPServerHeader, 	sServerHeaderPtr.Ptr, 		sServerHeaderPtr.Len);
	this->SetVal(qtssRTSPCurrentSessionCount, &fNumRTSPSessions, 		sizeof(fNumRTSPSessions));
	this->SetVal(qtssRTSPHTTPCurrentSessionCount, &fNumRTSPHTTPSessions,sizeof(fNumRTSPHTTPSessions));
	this->SetVal(qtssRTPSvrCurConn, 		&fNumRTPSessions, 			sizeof(fNumRTPSessions));
	this->SetVal(qtssRTPSvrTotalConn, 		&fTotalRTPSessions, 		sizeof(fTotalRTPSessions));
	this->SetVal(qtssRTPSvrCurBandwidth, 	&fCurrentRTPBandwidthInBits,sizeof(fCurrentRTPBandwidthInBits));
	this->SetVal(qtssRTPSvrTotalBytes, 		&fTotalRTPBytes, 			sizeof(fTotalRTPBytes));
	this->SetVal(qtssRTPSvrAvgBandwidth, 	&fAvgRTPBandwidthInBits, 	sizeof(fAvgRTPBandwidthInBits));
	this->SetVal(qtssRTPSvrCurPackets, 		&fRTPPacketsPerSecond, 		sizeof(fRTPPacketsPerSecond));
	this->SetVal(qtssRTPSvrTotalPackets, 	&fTotalRTPPackets, 			sizeof(fTotalRTPPackets));

	sServer = this;
}

#pragma mark __RTP_STATS_UPDATER_TASK__

RTPStatsUpdaterTask::RTPStatsUpdaterTask()
: 	Task(), fLastBandwidthTime(0), fLastBandwidthAvg(0), fLastBytesSent(0)
{
	this->Signal(Task::kStartEvent);
}

SInt64 RTPStatsUpdaterTask::Run()
{

	QTSServerInterface* theServer = QTSServerInterface::sServer;
	
	// All of this must happen atomically wrt dictionary values we are manipulating
	OSMutexLocker locker(&theServer->fMutex);
	
	//First update total bytes. This must be done because total bytes is a 64 bit number,
	//so no atomic functions can apply.
	//
	// NOTE: The line below is not thread safe on non-PowerPC platforms. This is
	// because the fPeriodicRTPBytes variable is being manipulated from within an
	// atomic_add. On PowerPC, assignments are atomic, so the assignment below is ok.
	// On a non-PowerPC platform, the following would be thread safe:
	//unsigned int periodicBytes = atomic_add(&theServer->fPeriodicRTPBytes, 0);
	unsigned int periodicBytes = theServer->fPeriodicRTPBytes;
	(void)atomic_sub(&theServer->fPeriodicRTPBytes, periodicBytes);
	theServer->fTotalRTPBytes += periodicBytes;
	
	// Same deal for packet totals
	unsigned int periodicPackets = theServer->fPeriodicRTPPackets;
	(void)atomic_sub(&theServer->fPeriodicRTPPackets, periodicPackets);
	theServer->fTotalRTPPackets += periodicBytes;
	
	SInt64 curTime = OS::Milliseconds();
	//also update current bandwidth statistic
	if (fLastBandwidthTime != 0)
	{
		Assert(curTime > fLastBandwidthTime);
		UInt32 delta = (UInt32)(curTime - fLastBandwidthTime);
		Assert(delta >= 1000);
		
		//do the bandwidth computation using floating point divides
		//for accuracy and speed.
		Float32 bits = (Float32)(periodicBytes * 8);
		Float32 theTime = (Float32)delta;
		theTime /= 1000;
		bits /= theTime;
		Assert(bits >= 0);
		theServer->fCurrentRTPBandwidthInBits = (UInt32)bits;
		
		//do the same computation for packets per second
		Float32 packetsPerSecond = (Float32)periodicPackets;
		packetsPerSecond /= theTime;
		Assert(packetsPerSecond >= 0);
		theServer->fRTPPacketsPerSecond = (UInt32)packetsPerSecond;
	}
	
	fLastBandwidthTime = curTime;
	
	//also compute average bandwidth, a much more smooth value. This is done with
	//the fLastBandwidthAvg, a timestamp of the last time we did an average, and
	//fLastBytesSent, the number of bytes sent when we last did an average.
	if ((fLastBandwidthAvg != 0) && (curTime > (fLastBandwidthAvg +
		(theServer->GetPrefs()->GetAvgBandwidthUpdateTimeInSecs() * 1000))))
	{
		UInt32 delta = (UInt32)(curTime - fLastBandwidthAvg);
		UInt64 bytesSent = theServer->fTotalRTPBytes - fLastBytesSent;

		//do the bandwidth computation using floating point divides
		//for accuracy and speed.
		Float32 bits = (Float32)(bytesSent * 8);
		Float32 theAvgTime = (Float32)delta;
		theAvgTime /= 1000;
		bits /= theAvgTime;
		Assert(bits >= 0);
		theServer->fAvgRTPBandwidthInBits = (UInt32)bits;

		fLastBandwidthAvg = curTime;
		fLastBytesSent = theServer->fTotalRTPBytes;
		
		//if the bandwidth is above the bandwidth setting, disconnect 1 user by sending them
		//a BYE RTCP packet.
		SInt32 maxKBits = theServer->GetPrefs()->GetMaxKBitsBandwidth();
		if ((maxKBits > -1) && (theServer->fAvgRTPBandwidthInBits > ((UInt32)maxKBits * 1024)))
		{
			//we need to make sure that all of this happens atomically wrt the session map
			OSMutexLocker locker(theServer->GetRTPSessionMap()->GetMutex());
			RTPSessionInterface* theSession = this->GetNewestSession(theServer->fRTPMap);
			if (theSession != NULL)
				if ((curTime - theSession->GetSessionCreateTime()) <
						theServer->GetPrefs()->GetSafePlayDurationInSecs() * 1000)
					theSession->Signal(Task::kKillEvent);
		}
	}
	else if (fLastBandwidthAvg == 0)
	{
		fLastBandwidthAvg = curTime;
		fLastBytesSent = theServer->fTotalRTPBytes;
	}
	
	(void)this->GetEvents();//we must clear the event mask!
	return theServer->GetPrefs()->GetTotalBytesUpdateTimeInSecs() * 1000;
}

RTPSessionInterface* RTPStatsUpdaterTask::GetNewestSession(OSRefTable* inRTPSessionMap)
{
	//Caller must lock down the RTP session map
	SInt64 theNewestPlayTime = 0;
	RTPSessionInterface* theNewestSession = NULL;
	
	//use the session map to iterate through all the sessions, finding the most
	//recently connected client
	for (OSRefHashTableIter theIter(inRTPSessionMap->GetHashTable()); !theIter.IsDone(); theIter.Next())
	{
		OSRef* theRef = theIter.GetCurrent();
		RTPSessionInterface* theSession = (RTPSessionInterface*)theRef->GetObject();
		Assert(theSession->GetSessionCreateTime() > 0);
		if (theSession->GetSessionCreateTime() > theNewestPlayTime)
		{
			theNewestPlayTime = theSession->GetSessionCreateTime();
			theNewestSession = theSession;
		}
	}
	return theNewestSession;
}

#pragma mark __PARAM_RETRIEVAL_FUNCTIONS__

//param retrieval functions described in .h file
void* QTSServerInterface::GetTotalUDPSockets(QTSS_Object inServer, UInt32* outLen)
{
	// This param retrieval function must be invoked each time it is called,
	// because the number of sockets can be continually changing
	QTSServerInterface* theServer = (QTSServerInterface*)inServer;
	// Multiply by 2 because this is returning the number of socket *pairs*
	theServer->fTotalUDPSockets = theServer->fSocketPool->GetSocketQueue()->GetLength() * 2;
	
	// Return the result
	*outLen = sizeof(theServer->fTotalUDPSockets);
	return &theServer->fTotalUDPSockets;
}

void* QTSServerInterface::IsOutOfDescriptors(QTSS_Object inServer, UInt32* outLen)
{
	// This param retrieval function must be invoked each time it is called,
	// because whether we are out of descriptors or not is continually changing
	QTSServerInterface* theServer = (QTSServerInterface*)inServer;
	
	theServer->fIsOutOfDescriptors = false;
	for (UInt32 x = 0; x < theServer->fNumListeners; x++)
	{
		if (theServer->fListeners[x]->IsOutOfDescriptors())
		{
			theServer->fIsOutOfDescriptors = true;
			break;
		}
	}
	// Return the result
	*outLen = sizeof(theServer->fIsOutOfDescriptors);
	return &theServer->fIsOutOfDescriptors;	
}
