/*
 * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * Copyright (c) 1999 Apple Computer, Inc.  All Rights Reserved.
 * The contents of this file constitute Original Code as defined in and are 
 * subject to the Apple Public Source License Version 1.1 (the "License").  
 * You may not use this file except in compliance with the License.  Please 
 * obtain a copy of the License at http://www.apple.com/publicsource and 
 * read it before using this file.
 * 
 * This Original Code and all software distributed under the License are 
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, 
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS 
 * FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the License for 
 * the specific language governing rights and limitations under the 
 * License.
 * 
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
/*
	File:		RTSPProtocol.h

	Contains:	A grouping of static utilities that abstract keyword strings
				in the RTSP protocol. This should be maintained as new versions
				of the RTSP protoocl appear & as the server evolves to take
				advantage of new RTSP features.

	$Log: RTSPProtocol.h,v $
	Revision 1.2  2000/02/11 01:07:23  serenyi
	Server now supports cache control header and sends 304 Not Modified responses in appropriate situations
	
	Revision 1.1.1.1  2000/01/17 23:36:03  wsanchez
	Import of StreamingServer-2.0 (Apple)
	
	Revision 1.2  1999/12/21 01:44:08  lecroy
	*** empty log message ***
	
	Revision 1.1.1.1  1999/09/22 23:01:53  lecroy
	importing
	
	Revision 1.1.1.1  1999/09/13 16:16:54  rtucker
	initial checkin from rrv14
	
	
	Revision 1.2  1999/02/19 23:08:31  ds
	Created
	

*/


#ifndef __RTSPPROTOCOL_H__
#define __RTSPPROTOCOL_H__

#include "QTSSRTSPProtocol.h"
#include "StrPtrLen.h"

class RTSPProtocol
{
	public:

		//METHODS
		
		//	Method enumerated type definition in QTSS_RTSPProtocol.h
			
		//The lookup function. Very simple
		static UInt32	GetMethod(const StrPtrLen &inMethodStr);
		
		//HEADERS

		//	Header enumerated type definitions in QTSS_RTSPProtocol.h
		
		//The lookup function. Very simple
		static UInt32 GetRequestHeader(const StrPtrLen& inHeaderStr);
		
		//The lookup function. Very simple.
		static StrPtrLen& GetHeaderString(UInt32 inHeader)
			{ return sHeaders[inHeader]; }
		
		
		//STATUS CODES

		//returns name of this error
		static StrPtrLen& 		GetStatusCodeString(QTSS_RTSPStatusCode inStat)
			{ return sStatusCodeStrings[inStat]; }
		//returns error number for this error
		static SInt32			GetStatusCode(QTSS_RTSPStatusCode inStat)
			{ return sStatusCodes[inStat]; }
		//returns error number as a string
		static StrPtrLen&		GetStatusCodeAsString(QTSS_RTSPStatusCode inStat)
			{ return sStatusCodeAsStrings[inStat]; }
		
		//VERSIONS
		enum RTSPVersion
		{
			k10Version = 0,
			kIllegalVersion = 1
		};
		
		//accepts strings that look like "RTSP/1.0" etc...
		static RTSPVersion		GetVersion(StrPtrLen &versionStr);
		static StrPtrLen&		GetVersionString(RTSPVersion version)
			{ return sVersionString[version]; }
		
	private:

		//for other lookups
		static StrPtrLen			sMethods[];
		static StrPtrLen			sHeaders[];
		static StrPtrLen			sStatusCodeStrings[];
		static StrPtrLen			sStatusCodeAsStrings[];
		static SInt32				sStatusCodes[];
		static StrPtrLen 			sVersionString[];
		static SInt8				sMapStatusCodeToLogString[];

};
#endif // __RTSPPROTOCOL_H__
