/*
 * Copyright (c) 1999 Apple Computer, Inc. All rights reserved.
 *
 * @APPLE_LICENSE_HEADER_START@
 * 
 * Copyright (c) 1999 Apple Computer, Inc.  All Rights Reserved.
 * The contents of this file constitute Original Code as defined in and are 
 * subject to the Apple Public Source License Version 1.1 (the "License").  
 * You may not use this file except in compliance with the License.  Please 
 * obtain a copy of the License at http://www.apple.com/publicsource and 
 * read it before using this file.
 * 
 * This Original Code and all software distributed under the License are 
 * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER 
 * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES, 
 * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY, FITNESS 
 * FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the License for 
 * the specific language governing rights and limitations under the 
 * License.
 * 
 * 
 * @APPLE_LICENSE_HEADER_END@
 */
/*
	File:		main.cpp

	Contains:	main function to drive streaming server.

	$NoKeywords: $
	
	$Log: main.cpp,v $
	Revision 1.5  2000/03/17 01:11:01  serenyi
	Solaris has low max # of fds by default, so manually set limit higher
	
	Revision 1.4  2000/03/07 21:11:57  rtucker
	changes for Config file commonization
	
	Revision 1.3  2000/01/28 08:31:15  serenyi
	Fixed startup sequence so RTCPs get processed
	
	Revision 1.2  2000/01/27 02:09:14  rtucker
	Merge changes from v81
	
	Revision 1.21  2000/01/26 18:33:26  serenyi
	#2426075 Integrated changes from Kostya Vasilyev
	
	Revision 1.20  2000/01/21 23:52:56  rtucker
	
	logging changes
	
	Revision 1.19  2000/01/19 19:37:06  serenyi
	Moved to new NEW macro
	
	Revision 1.18  1999/12/21 01:44:11  lecroy
	*** empty log message ***
	
	Revision 1.17  1999/12/14 21:31:01  lecroy
	various QT -> Darwin name changes, fixed up Installit
	
	Revision 1.16  1999/12/02 19:26:27  serenyi
	Got rid of "RoadRunner", added a platform string to the Server: header
	
	Revision 1.15  1999/12/02 04:39:29  murata
	Added daemon call for solaris. Linked in Make.
	
	Revision 1.14  1999/11/15 01:20:49  serenyi
	Added 4-char codes for mem debugging, merged in solaris diffs
	
	Revision 1.13  1999/11/08 02:27:53  lecroy
	long long printf macro & sgi mods
	
	Revision 1.12  1999/11/02 04:03:19  serenyi
	Don't call sleep on MacOS!
	
	Revision 1.11  1999/10/30 07:21:27  serenyi
	made work on linux
	
	Revision 1.10  1999/10/29 22:07:00  rtucker
	changed "contains" description
	
	Revision 1.9  1999/10/28 20:58:16  serenyi
	Added SIGHUP handler in one place it needed to be added
	
	Revision 1.8  1999/10/27 17:55:05  serenyi
	Integrated Matthias's MacOS / GUSI changes
	
	Revision 1.7  1999/10/27 03:28:22  lecroy
	merged in changes from Akamai
	
	Revision 1.6  1999/10/25 14:43:02  rtucker
	redeced warning count
	
	Revision 1.5  1999/10/21 23:55:19  serenyi
	Got rid of -f from reported list of options
	
	Revision 1.4  1999/10/07 08:52:34  serenyi
	Changed bool to Bool16, changed QTSS_Dictionary to QTSS_Object
	
	Revision 1.3  1999/10/01 19:48:34  rtucker
	change threads to one for alpha seed
	
	Revision 1.2  1999/09/30 00:33:51  serenyi
	Added command line option for base64 decode
	
	Revision 1.1.1.1  1999/09/22 23:01:49  lecroy
	importing
	
	Revision 1.4  1999/09/15 22:33:26  rtucker
	-f option is for fast start from /etc/ streamingserver.conf
	
	Revision 1.3  1999/09/15 17:06:18  serenyi
	Added SIGHUP reread preferences support
	
	Revision 1.2  1999/09/14 21:29:53  rtucker
	in debug mode, QTSS reads ./streamingserver.conf for config info
	when pass the -d flag.  Does not override a -c setting.
	
	Revision 1.1.1.1  1999/09/13 16:16:52  rtucker
	initial checkin from rrv14
	
	
	Revision 1.2  1999/02/19 23:15:47  ds
	Created
	

*/

#include <stdio.h>
#include <stdlib.h>

#if __PTHREADS__
#include <pthread.h>
#else
#include <mach/mach.h>
#include <mach/cthreads.h>
#endif

#include <sys/time.h>
#include <fcntl.h>
#include <signal.h>
#include <unistd.h>
#include "ev.h"
#include <sys/errno.h>

#ifndef __MACOS__
#include <sys/resource.h>
#include <sys/wait.h>
#endif

#ifdef __solaris__	
#include "daemon.h"
#endif

#include "QTSServer.h"
#include "QTSServerInterface.h"
#include "QTSSExpirationDate.h"
#include "RTSPSessionInterface.h"
#include "Socket.h"
#include "OSMemory.h"
#include "MyAssert.h"

#include "SocketUtils.h"
#include "OSCodeFragment.h"
#include "IdleTask.h"
#include "TimeoutTask.h"
#include "OS.h"

#include "PrefsSource.h"
#include "FilePrefsSource.h"
#if __MacOSX__
#include "NetInfoPrefsSource.h"
#endif

void sigcatcher(int sig, int /*sinfo*/, struct sigcontext* /*sctxt*/);

void sigcatcher(int sig, int /*sinfo*/, struct sigcontext* /*sctxt*/)
{
#if DEBUG
	//printf("Signal %d caught\n", sig);
#endif
	if (sig == SIGHUP)
		(void)QTSServer::RereadPrefsService(NULL);
}

extern "C" {
typedef int (*EntryFunction)(int input);
}

//This function runs the server
void RunServer(PrefsSource* inPrefsSource, PrefsSource* inMessagesSource, UInt16 inPortOverride);


int main(int argc, char * argv[]) 
{
	extern char* optarg;
	
	// on write, don't send signal for SIGPIPE, just set errno to EPIPE
	// and return -1
	//signal is a deprecated and potentially dangerous function
	//(void) ::signal(SIGPIPE, SIG_IGN);
	struct sigaction act;
	
#if defined(sun) || defined(i386)
	sigemptyset(&act.sa_mask);
	act.sa_flags = 0;
	act.sa_handler = (void(*)(int))&sigcatcher;
#else
	act.sa_mask = 0;
	act.sa_flags = 0;
	act.sa_handler = (void(*)(...))&sigcatcher;
#endif
	(void)::sigaction(SIGPIPE, &act, NULL);
	(void)::sigaction(SIGHUP, &act, NULL);

#if __MacOSX__ || __solaris__
	//grow our pool of file descriptors to the max!
	struct rlimit rl;
               
	rl.rlim_cur = Socket::kMaxNumSockets;
	rl.rlim_max = Socket::kMaxNumSockets;

	setrlimit (RLIMIT_NOFILE, &rl);
#endif
	
	//First thing to do is to read command-line arguments and initialize the pipe
	//file descriptor. This is used for returning errors back to whatever is starting
	//the process in the event of a fatal error very early in the startup process
	int thePipe = -1; 
	int ch;
	UInt16 thePort = 0; //port can be set on the command line
	
	Bool16 dontFork = false;
	char* theConfigFilePath = NULL;
#ifndef __MacOSX__
#if __MACOS__
	static char* sDefaultConfigFilePath = "streamingserver.conf";
#else
	static char* sDefaultConfigFilePath = "/etc/streamingserver.conf";
#endif
	//Other platforms don't have the option of using NetInfo for preferences,
	//so by default the server will grab prefs from a config file
	theConfigFilePath = sDefaultConfigFilePath;
#endif


	
#ifndef __MACOS__
	while ((ch = getopt(argc,argv, "vdZbfp:c:")) != EOF)
	{
		switch(ch)
		{
			case 'v':
				printf("%s/%s-%s Built on: %s\n", 	QTSServerInterface::GetServerName().Ptr,
													QTSServerInterface::GetServerVersion().Ptr,
													QTSServerInterface::GetServerPlatform().Ptr,
													QTSServerInterface::GetServerBuildDate().Ptr);
				printf("-d: Run in the foreground\n");
				//printf("-f: Use config file at /etc/streamingserver.conf\n");
				printf("-p XXX: Specify the default RTSP listening port of the server\n");
				printf("-c /myconfigpath.conf: Specify a config file\n");
				exit(0);	
			case 'Z':
				//this option is passed in by server control lib when it
				//starts the server. Immediately following is a path to the piped file descriptor
				thePipe = open(argv[2], O_WRONLY);
				break;
			case 'd':
				dontFork = true;
				
				break;
			case 'f':
				theConfigFilePath  = "/etc/streamingserver.conf";
				break;
			case 'p':
				thePort = ::atoi(optarg);
				break;
			case 'c':
				theConfigFilePath = optarg;
				break;
			default:
				break;
		}
	}
	
	// Check expiration date
	QTSSExpirationDate::PrintExpirationDate();
	if (QTSSExpirationDate::IsSoftwareExpired())
	{
		printf("Streaming Server has expired\n");
		exit(0);
	}

	//Unless the command line option is set, fork & daemonize the process at this point
	if (!dontFork)
	{
#ifdef __sgi
		// for some reason, this method doesn't work right on IRIX 6.4 unless the first arg
		// is _DF_NOFORK.  if the first arg is 0 (as it should be) the result is a server
		// that is essentially paralized and doesn't respond to much at all.  So for now,
		// leave the first arg as _DF_NOFORK
		if (_daemonize(_DF_NOFORK, STDIN_FILENO, STDOUT_FILENO, STDERR_FILENO) != 0)
#else
		if (daemon(0,0) != 0)
#endif
		{
#if DEBUG
			printf("Failed to daemonize process. Error = %d\n", OSThread::GetErrno());
#endif
			exit(-1);
		}
	}
#endif //#ifndef __MACOS__

	//Construct a Prefs Source object to get server preferences
	PrefsSource* thePrefsSource = NULL;
	
	if (theConfigFilePath != NULL)
	{	
		FilePrefsSource *filePrefs = new FilePrefsSource(true);
		int 	configErr = filePrefs->InitFromConfigFile(theConfigFilePath);
		
		thePrefsSource = filePrefs;	// cast to PrefsSource for later use
	
		if ( configErr )
		{
			printf("Fatal Error: Could not load configuration file at %s. (%d)\n", theConfigFilePath, OSThread::GetErrno());
			exit(-1);
		}
	}
#if __MacOSX__
	else
		thePrefsSource = new NetInfoPrefsSource();
#endif
	Assert(thePrefsSource != NULL);//we must have one in order to function
	
	//Construct a Prefs Source object to get server text messages
	FilePrefsSource theMessagesSource;
	theMessagesSource.InitFromConfigFile("qtssmessages.txt");
	
	//check if we should do auto restart. If so, we should fork the process at this point,
	//have the child run the server. Parent will just wait for the child to die.
	char autoStartSetting[256] = "";
	(void)thePrefsSource->GetValue( "auto_restart", autoStartSetting);
	
#ifndef __MACOS__
	
	int statloc = 0;
	if ((!dontFork) && (::strcmp(autoStartSetting, "enabled") == 0))
	{
		//loop until the server exits normally. If the server doesn't exit
		//normally, then restart it.
		while (true)
		{
			pid_t processID = fork();
			Assert(processID >= 0);
			if (processID > 0)
			{
				::wait(&statloc);
				
#if DEBUG
				::printf("Child Process %d exited with status %d\n", processID, statloc);
#endif
				if (statloc == 0)
					return 0;
			}
			else if (processID == 0)
				break;
			else
				exit(-1);
			//eek. If you auto-restart too fast, you might start the new one before the OS has
			//cleaned up from the old one, resulting in startup errors when you create the new
			//one. Waiting for a second seems to work
			sleep(1);
		}
	}
#endif //#ifndef __MACOS__
	
	//we have to do this again for the child process, because sigaction states
	//do not span multiple processes.
	(void)::sigaction(SIGPIPE, &act, NULL);
	(void)::sigaction(SIGHUP, &act, NULL);

	//This function starts, runs, and shuts down the server
	RunServer(thePrefsSource, &theMessagesSource, thePort);
}

void RunServer(PrefsSource* inPrefsSource, PrefsSource* inMessagesSource, UInt16 inPortOverride)
{
	//Mark when we are done starting up. If auto-restart is enabled, we want to make sure
	//to always exit with a status of 0 if we encountered a problem WHILE STARTING UP. This
	//will prevent infinite-auto-restart-loop type problems
	Bool16 doneStartingUp = false;
	
	//Initialize utility classes
	OS::Initialize();
	Socket::Initialize();
	SocketUtils::Initialize();

#if !MACOSXEVENTQUEUE
	::select_startevents();//initialize the select() implementation of the event queue
#endif
	
	//start the server
	QTSSModuleInterface::Initialize();
	QTSServerInterface::Initialize();// this must be called before constructing the server object
	QTSServer* server = NEW QTSServer();
	server->Initialize(inPrefsSource, inMessagesSource, inPortOverride);

	//Make sure to do this stuff last. Because these are all the threads that
	//do work in the server, this ensures that no work can go on while the server
	//is in the process of staring up
	if (server->GetServerState() != qtssFatalErrorState)
	{
		const UInt32 kNumTaskThreads = 1;
		TaskThreadPool::AddThreads(kNumTaskThreads);
	#if DEBUG
		printf("Number of task threads: %lu\n",kNumTaskThreads);
	#endif
		IdleTask::Initialize();
		TimeoutTask::Initialize();
		Socket::StartThread();
		
		// Start up the server's global tasks
		server->StartTasks();
	}

	if (server->GetServerState() != qtssFatalErrorState)
	{
		doneStartingUp = true;
		printf("Streaming Server done starting up\n");
		OSMemory::SetMemoryError(ENOMEM);
	}
	
#if __MACOS__
	OSMutex theMutex;
	OSCond theCond;
#endif
	
	//just wait until someone stops the server or a fatal error occurs.
	QTSS_ServerState theServerState = server->GetServerState();
	while ((theServerState != qtssShuttingDownState) &&
			(theServerState != qtssFatalErrorState))
	{
#if __MACOS__
		theMutex.Lock();
		theCond.Wait(&theMutex, 10000);
		theMutex.Unlock();
#else
		sleep(1);
#endif
		theServerState = server->GetServerState();
	}
		
	//first off, make sure that the server can't do any work
	TaskThreadPool::RemoveThreads();
	
	//now that the server is definitely stopped, it is safe to initate
	//the shutdown process
	delete server;
	
	//ok, we're ready to exit. If we're quitting because of some fatal error
	//while running the server, make sure to let the parent process know by
	//exiting with a nonzero status. Otherwise, exit with a 0 status
	if ((doneStartingUp) && (theServerState == qtssFatalErrorState))
		exit (-1);//signals parent process to restart server
}
